<?php
namespace Teknisa\FIS\Modules\Acessorias\Controllers;

use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request\Filter;
use Zeedhi\Framework\DTO;
use Teknisa\FIS\Modules\Acessorias\Services\Fis75000EfdReinfService;
use Teknisa\FIS\Utilities\MessagesFiscal;
use Teknisa\Libs\Util\Masks\Masks as MasksService;

 
class Fis75000EfdReinfController extends \Teknisa\FIS\Controllers\TeknisaCrud {
    
    // Parametros de Rotas
    const IDTRANSMISSAO = 'idTransmissao';

    // Operações do Reinf
    const IDTRANSENVIO = 'ENVIAR'; //Enviar Evento
    const IDTRANSCONSU = 'CONSULTAR'; //Consultar Status do evento
    const IDTRANSEXCLU = 'EXCLUSAO'; //Excluir Evento
    //const IDTRANSREENV = 'REENVIAR'; //Reenviar Evento
    
    // Nomes dos Eventos
    const R1000 = 'R-1000';
    const R1070 = 'R-1070';
    const R2010 = 'R-2010';
    const R2020 = 'R-2020';
    const R2060 = 'R-2060';
    const R2098 = 'R-2098';
    const R2099 = 'R-2099';
    const R9000 = 'R-9000';
    
    protected $service;
    protected $crudService;
    protected $dataSourceName = 'envioeventoreinf';
    protected $dataSourceNameToSave  = false;
    protected $parameterBag;
    protected $parameterBagColumns  = array();

    public function __construct($crudService, ParameterBag $parameterBag, Fis75000EfdReinfService $service) {
        parent::__construct($crudService, $parameterBag);
        $this->service = $service;
        set_time_limit(0);
    }
    
    // --------------------  Funções de CRUD
    public function find(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = 'envioeventoreinf_eventoreinf';
        parent::find($request, $response);
        $newDataSets = array();
        $dataSets = $response->getDataSets();
        foreach($dataSets as $dataSet) {
            $maskService = new MasksService;
            $newRows = array();
            foreach($dataSet->getRows() as $row) {
                if (isset($row['CDCLIFORN'])) {
                    $row['CDCLIFORN'] = $maskService->mask($row['CDCLIFORN'], $maskService->picturePessoa($row['TIPOINSCRICAO']));
                }
                $newRows[] = $row;
            }
            $dataSets[] = new DataSet('envioeventoreinf_eventoreinf', $newRows);
        };
        $response->setDataSets($dataSets);
    }
    
    public function findEventosExclusao(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = 'enventos_para_exclusao';
        parent::find($request, $response);
        $newDataSets = array();
        $dataSets = $response->getDataSets();
        foreach($dataSets as $dataSet) {
            $maskService = new MasksService;
            $newRows = array();
            foreach($dataSet->getRows() as $row) {
                if (isset($row['CDCLIFORN'])) {
                    $row['CDCLIFORN'] = $maskService->mask($row['CDCLIFORN'], $maskService->picturePessoa($row['TIPOINSCRICAO']));
                }
                $row['DTMESCOMPETENC'] = substr($row['DTMESCOMPETENC'], 0, 10);
                $newRows[] = $row;
            }
            $dataSets[] = new DataSet('enventos_para_exclusao', $newRows);
        };
        $response->setDataSets($dataSets);
    }

    public function findR1000Vigente(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $empresa = $conditions[0]['value'];
            $ambiente = $conditions[1]['value'];
            $dtinicial = $conditions[2]['value'];
            $dtfinal = $conditions[3]['value'];
            
            $nrenvio = $this->service->nrenvioR1000vigente($empresa, $ambiente, $dtinicial, $dtfinal);
            $dataSet = new DataSet('R1000_vigente', array("NRENVIO" => $nrenvio));
    
            $response->setDataSets(array($dataSet));
        } catch (\Exception $e) {
            if ($e->getMessage() === MessagesFiscal::R1000_INEXISTENTE) {
                $response->setDataSets(array(new DataSet('R1000_vigente', array("NRENVIO" => null))));
            } else {
                $response->setError(new Error($e->getMessage(), 500));
            }
        }
        
    }  
       
	// Função que realizara a busca de todas as empresas, aplicando a máscara da inscrição
    public function findEmpresa(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $this->dataSourceName = 'empresa';
            parent::find($request, $response);
            $dataSets = $response->getDataSets();

            if(!empty($dataSets)) {
                $rows    = $dataSets[0]->getRows();
                $dados   = $this->service->aplicarMascaraInscricao($rows,'NRINSJUREMPR');
                $dataSet = new DataSet("empresa_mascaraInscricao", $dados);
                $response->setDataSets(array($dataSet));
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    // -------------------  Funções de operações
    public function salvarEvento(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $rows       = $this->crudService->getRowsToSave($request, 'envioeventoreinf')[0];
            $msgRetorno = $this->service->salvarEvento($rows);
            $response->addDataSet(new DataSet("columnsToReturn", $this->crudService->getColumnsToReturn()));
            $response->addMessage(new DTO\Response\Message($msgRetorno));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function getParametrosReinf(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row        = $request->getRow();
            $parametros = $this->service->getParamReinf($row['CDEMPRESA']);
            $response->addDataSet(new DataSet("parametros", $parametros));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }   
    
    public function detalhamentoR2060(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row        = $request->getRow();
            $detalhamentoR2060 = $this->service->detalhamentoR2060($row);
            $response->addDataSet(new DataSet("detalhamentoR2060", $detalhamentoR2060));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }   
    
    public function setTransmissaoReinf(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            
            $oldmask       = umask(0);
            $msgRetorno    = '';
            $idTransmissao = $request->getParameter(self::IDTRANSMISSAO);
            $idGrupo       = '';
            $dsVersao      = '';

            $this->service->entityManager->beginTransaction();
            //$paramReinf = $this->service->getParamReinf($request->getDataSet()->getRows()[0]['CDEMPRESA']);
            $firstRow = $request->getDataSet()->getRows()[0];
            
            $this->service->setDadosTransmissao($firstRow);
           //  var_dump('oii');die;
            $eventosLote = array();
            if ($idTransmissao == self::IDTRANSENVIO /*|| $idTransmissao == self::IDTRANSREENV*/) {
                
                foreach ($request->getDataSet()->getRows() as &$row) {
                    
                    $idGrupo   = $row['IDGRUPO'];
                    $nmevento  =  $row['NMEVENTO'];
                    $dsVersao  = str_replace('.','-',$row['DSVERSAO']);
                    $notasItemSemServico = $this->service->validaServicoDosItems($row, $nmevento);
                    if (count($notasItemSemServico) > 0) {
                        $response->addMessage(new Message(MessagesFiscal::notasItensSemServico($notasItemSemServico), 'W'));
                        return ; 
                    }
                    
                    $xmlEvento = $this->service->getXmlEvento($dsVersao, $row);
                  //  var_dump($xmlEvento);die;
                    if ($xmlEvento) {
                        $eventosLote[] = $xmlEvento;
                    }
                }
              //  var_dump('oii');die;
                if (empty($eventosLote)) {
                    throw new \Exception(MessagesFiscal::DADOS_LOTE_VAZIO,1);
                }
                
                $xmlLoteEventos = $this->service->getXmlLote($idGrupo, $firstRow, $dsVersao, $eventosLote);
                
                if($firstRow["IDOPERACAO"] == "R"){
                    $this->service->updateVersaoRetificacao($firstRow["NRENVIO"], $firstRow["NMEVENTO"], $firstRow["DSVERSAO"]);
                }
                
                $retorno = $this->service->enviarLote($eventosLote, $xmlLoteEventos);
                $msgRetorno = $retorno['MSG'];
            } else if ($idTransmissao == self::IDTRANSCONSU) {
                // var_dump('oii');die;
                if($firstRow['NMEVENTO'] === self::R2099){    
                    if($firstRow['TOTALIZADOR'] == true){ 
                        $xmlConsulta = $this->service->geraXmlConsulta($firstRow['NRENVIO']);
                        $retorno = $this->service->enviarLote($firstRow, $xmlConsulta);
                        $msgRetorno = $retorno['MSG'];
                        
                        if (isset($retorno['REPORT_URL'])) {
                            $response->setDataSets(array(new DataSet('report', array('report_url' => $retorno['REPORT_URL']))));
                        }
                    } else {
                        $xmlConsulta[0] = $this->service->geraXmlConsulta2098($firstRow);
                        $xmlConsulta[1] = $this->service->geraXmlConsulta2099($firstRow);
                        $firstRow['NMEVENTO'] = 'R-2098'; 
                        $retorno = $this->service->enviarLoteConsulta($firstRow, $xmlConsulta[0], true);

                        if(($retorno['COD'] != '1') || ($retorno['COD'] == '1' && $firstRow['IDSTATUS'] == 'C' )){
                            $firstRow['NMEVENTO'] = 'R-2099';
                            $retorno = $this->service->enviarLoteConsulta($firstRow, $xmlConsulta[1]);
                            $msgRetorno = $retorno['MSG'];
                        } else if ($retorno['COD'] == '1' && $firstRow['IDSTATUS'] == 'P' ){
                            $firstRow['NMEVENTO'] = 'R-2099';
                            $retorno = $this->service->enviarLoteConsulta($firstRow, $xmlConsulta[1], true);
                            $msgRetorno = $retorno['MSG'];
                        }
                        
                        if (isset($retorno['REPORT_URL'])) {
                            $response->setDataSets(array(new DataSet('report', array('report_url' => $retorno['REPORT_URL']))));
                        }    
                    }
                } 
                
                else if($firstRow['NMEVENTO'] === self::R1000){
               // var_dump('oii');die;
                    $xmlConsulta = $this->service->geraXmlConsulta1000($firstRow);
                    
                    $retorno = $this->service->enviarLoteConsulta($firstRow, $xmlConsulta);
                    $msgRetorno = $retorno['MSG'];
                    
                    if (isset($retorno['REPORT_URL'])) {
                        $response->setDataSets(array(new DataSet('report', array('report_url' => $retorno['REPORT_URL']))));
                    }
                    
                } 
                
                else if($firstRow['NMEVENTO'] === self::R2010){
                    $xmlConsulta = $this->service->geraXmlConsulta2010($firstRow);
                    
                    $retorno = $this->service->enviarLoteConsulta($firstRow, $xmlConsulta);
                    $msgRetorno = $retorno['MSG'];
                    
                    if (isset($retorno['REPORT_URL'])) {
                        $response->setDataSets(array(new DataSet('report', array('report_url' => $retorno['REPORT_URL']))));
                    }
                    
                } 
                
                else if($firstRow['NMEVENTO'] === self::R2020){
                    $xmlConsulta = $this->service->geraXmlConsulta2020($firstRow);
                    
                    $retorno = $this->service->enviarLoteConsulta($firstRow, $xmlConsulta);
                    $msgRetorno = $retorno['MSG'];
                    
                    if (isset($retorno['REPORT_URL'])) {
                        $response->setDataSets(array(new DataSet('report', array('report_url' => $retorno['REPORT_URL']))));
                    }
                    
                } 
                
                else if($firstRow['NMEVENTO'] === self::R2060){
                    $xmlConsulta = $this->service->geraXmlConsulta2060($firstRow);
                    
                    $retorno = $this->service->enviarLoteConsulta($firstRow, $xmlConsulta);
                    $msgRetorno = $retorno['MSG'];
                    
                    if (isset($retorno['REPORT_URL'])) {
                        $response->setDataSets(array(new DataSet('report', array('report_url' => $retorno['REPORT_URL']))));
                    }
                }
                
                else if($firstRow['NMEVENTO'] === self::R2098){
                    $xmlConsulta = $this->service->geraXmlConsulta2098($firstRow);
                    
                    $retorno = $this->service->enviarLoteConsulta($firstRow, $xmlConsulta);
                    $msgRetorno = $retorno['MSG'];
                    
                    if (isset($retorno['REPORT_URL'])) {
                        $response->setDataSets(array(new DataSet('report', array('report_url' => $retorno['REPORT_URL']))));
                    }
                }
                
            } else if ($idTransmissao == self::IDTRANSEXCLU) {
                foreach ($request->getDataSet()->getRows() as &$row) {
                    
                    $idGrupo   = $row['IDGRUPO'];
                    $nmevento  =  $row['NMEVENTO'];
                    $dsVersao  = str_replace('.','-',$row['DSVERSAO']);
                    $xmlEvento = $this->service->getXmlEvento($dsVersao, $row);
                    if ($xmlEvento) {
                        $eventosLote[] = $xmlEvento;
                    }
                }
                
                $xmlLoteEventos = $this->service->getXmlLote($idGrupo, $firstRow, $dsVersao, $eventosLote);
                $retorno = $this->service->enviarLote($eventosLote, $xmlLoteEventos);
                $msgRetorno = $retorno['MSG'];
            }

            if ($msgRetorno) {
                $this->service->entityManager->flush();
                $this->service->entityManager->commit();
                $response->addMessage(new Message($msgRetorno));
            } else {
                $this->service->entityManager->rollback();
                $response->addMessage(new Message("Serviço '$idTransmissao' não disponível", Message::TYPE_WARNING));
            }

            umask($oldmask);
        } catch(\Exception $e) {
            $this->service->entityManager->rollback();
            if ($e->getCode() == MessagesFiscal::ALERT_MESSAGE_CODE || $e->getCode() == MessagesFiscal::ERROR_MESSAGE_CODE) {
                $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            } else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
            }
        }
    }
    
    public function saveItemSemParam(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        if($row["ALLINSC"] == 'Y') {
            $rowsToSave =  array();
            $inscricoes = $this->service->getInscricoesMunicipais($row["CDEMPRESA"]);
            $newCode    = $this->service->getNewCode();
            foreach ($inscricoes as $inscricao) {
                $row["CDINSCMUNI"] = $inscricao["CDINSCMUNI"];
                
                $this->crudService->save('itemsemparametrizacao', array($row));
            }   
        }
    }
    
    public function limparBase(DTO\Request\Row $request, DTO\Response $response){
        try {
            $msgRetorno    = '';
            $idGrupo       = '';
            $dsVersao      = '';

            $this->service->entityManager->beginTransaction();

            $paramReinf = $this->service->getParamReinf($request->getRow()['CDEMPRESA']);
            
            if($paramReinf[0]['IDAMBIENTE'] === '2'){
            
                $dadosR1000LB = $this->service->getDadosR1000LimparBase($request->getRow()['CDEMPRESA']);
                
                $getEventoR1000 = $this->service->getEventoR1000($paramReinf[0]['CDVERSAO']);
                
                $dadosR1000LB = array_merge($dadosR1000LB, $getEventoR1000);
                
                $dadosR1000LB['IDOPERACAO'] = 'I';
                $dadosR1000LB['NRENVIO'] = null;
                $dadosR1000LB['DTINIVALIDADE'] = null;
                $dadosR1000LB['DTFINVALIDADE'] = null;
                $dadosR1000LB['DSDIRXMLENV'] = null;
                $dadosR1000LB['IDEVENTO'] = null;
                $dadosR1000LB['IDAMBIENTE'] = $paramReinf[0]['IDAMBIENTE'];
                
                $this->service->setDadosTransmissao($dadosR1000LB);
                
                $eventosLote = array();
                        
                $idGrupo   = $dadosR1000LB['IDGRUPO'];
                $nmevento  =  $dadosR1000LB['NMEVENTO'];
                $dsVersao  = str_replace('.','-',$dadosR1000LB['DSVERSAO']);
                
                $notasItemSemServico = $this->service->validaServicoDosItems($dadosR1000LB, $nmevento);
                if (count($notasItemSemServico) > 0) {
                    $response->addMessage(new Message(MessagesFiscal::notasItensSemServico($notasItemSemServico), 'W'));
                    return ; 
                }
                $xmlEvento = $this->service->getXmlEventoLimparBase($dsVersao, $dadosR1000LB);
                
                if ($xmlEvento) {
                    $eventosLote[] = $xmlEvento;
                }
            
                if (empty($eventosLote)) {
                    throw new \Exception(MessagesFiscal::DADOS_LOTE_VAZIO,1);
                }
                
                $paramReinf = array_merge($dadosR1000LB, $paramReinf);
                
                $xmlLoteEventos = $this->service->getXmlLote($idGrupo, $paramReinf, $dsVersao, $eventosLote);
                
                $retorno = $this->service->enviarLimparBase($eventosLote, $xmlLoteEventos, $request->getRow()['CDEMPRESA'], $dadosR1000LB['IDAMBIENTE']);
                
                $msgRetorno = $retorno['MSG'];
                
                if ($msgRetorno) {
                    $this->service->entityManager->flush();
                    $this->service->entityManager->commit();
                    $response->addMessage(new Message($msgRetorno));
                } else {
                    $this->service->entityManager->rollback();
                    $response->addMessage(new Message("Serviço '$idTransmissao' não disponível", Message::TYPE_WARNING));
                }
            } 
            else {
                throw new \Exception('Só é possivel limpar a base para: 2 - Ambiente de produção restrita',0);
            }
        } catch(\Exception $e) {
            $this->service->entityManager->rollback();
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

}