<?php

namespace Teknisa\FIS\Modules\Acessorias\Services;

use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DataSource\Manager\SQL\ManagerImpl as DataSourceManager;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Row;
use Zeedhi\Framework\Report\ReportService;
use Teknisa\FIS\Modules\Acessorias\Factories\Fis75000EfdReinfFactory;
use Teknisa\FIS\Modules\Acessorias\Utilities\Template;
use Teknisa\FIS\Utilities\MessagesFiscal;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\Utilities;
use Teknisa\Libs\Util\Masks\Masks;
use Teknisa\Libs\Util\DateUtil;
use Teknisa\Libs\Util\NewCode;
use DateTime;
//Entities
use Teknisa\Model\Fiscal\Entities\Envioeventoreinf;
use Teknisa\Model\Fiscal\Entities\Eventoreinf;
use Teknisa\Model\Fiscal\Entities\Paramempreinf;
use Teknisa\Model\Geral\Entities\Empresa;
use Zeedhi\Framework\Remote\FileUploader;
//use Zeedhi\FileServerAPI\API;

class Fis75000EfdReinfService {
    
    const TPAMB_PROD           = 1;
    const TPAMB_PROD_REST_REAL = 2;
    const TPAMB_PROD_REST_FICT = 3;

    const IDSTATUS_PENDENTE           = 'P';
    const IDSTATUS_ERRO_ENVIO         = 'E';
    const IDSTATUS_PROCESSAMENTO      = 'M';
    const IDSTATUS_ERRO_PROCESSAMENTO = 'R';
    const IDSTATUS_CONCLUIDO          = 'C';
    
    // Nomes dos Eventos
    const R1000 ='R-1000';
    const R1070 ='R-1070';
    const R2010 ='R-2010';
    const R2020 ='R-2020';
    const R2060 ='R-2060';
    const R2098 ='R-2098';
    const R2099 ='R-2099';
    const R9000 ='R-9000';
    
    // Nomes relatórios
    const R2099REPORT = 'CONSULTAR2099';


    /* Códigos de retorno do envio do lote */
    const CD_SUCESSO = '0';
    const CD_ERRO    = '1';
    const CD_EM_PROCESS = '2';
    
    const ERRO        = 'Erro';
    const ADVERTENCIA = 'Advertência';
    const VERSAO_ATUAL = '1.3';

    const ENVIAR_LOTE_PROD = 'https://reinf.receita.fazenda.gov.br/WsREINF/RecepcaoLoteReinf.svc';
    const ENVIAR_LOTE_TEST = 'https://preprodefdreinf.receita.fazenda.gov.br/WsREINF/RecepcaoLoteReinf.svc';
    const SOAPACTION_ENVIO_LOTE = 'http://sped.fazenda.gov.br/RecepcaoLoteReinf/ReceberLoteEventos';
    
    // Mudança do WebService publicado em 15/01/2019
    //const CONSULTA_LOTE_PROD = "https://reinf.receita.fazenda.gov.br/WsREINF/ConsultasReinf.svc";
    //const CONSULTA_LOTE_TEST = "https://preprodefdreinf.receita.fazenda.gov.br/WsREINF/ConsultasReinf.svc";
    
    const CONSULTA_LOTE_PROD = "https://reinf.receita.fazenda.gov.br/WsReinfConsultas/ConsultasReinf.svc";
    const CONSULTA_LOTE_TEST = "https://preprodefdreinf.receita.fazenda.gov.br/WsReinfConsultas/ConsultasReinf.svc";
    const SOAPACTION_CONSULTA = 'http://sped.fazenda.gov.br/ConsultasReinf/ConsultaResultadoFechamento2099';
    const SOAPACTION_CONSULTA_R1000 = 'http://sped.fazenda.gov.br/ConsultasReinf/ConsultaReciboEvento1000';
    const SOAPACTION_CONSULTA_R2010 = 'http://sped.fazenda.gov.br/ConsultasReinf/ConsultaReciboEvento2010';
    const SOAPACTION_CONSULTA_R2020 = 'http://sped.fazenda.gov.br/ConsultasReinf/ConsultaReciboEvento2020';
    const SOAPACTION_CONSULTA_R2060 = 'http://sped.fazenda.gov.br/ConsultasReinf/ConsultaReciboEvento2060';
    const SOAPACTION_CONSULTA_R2098 = 'http://sped.fazenda.gov.br/ConsultasReinf/ConsultaReciboEvento2098';
    const SOAPACTION_CONSULTA_R2099 = 'http://sped.fazenda.gov.br/ConsultasReinf/ConsultaReciboEvento2099';
     
    public    $entityManager;
    protected $dataSourceManager;
    protected $environment;
    protected $factory;
    protected $masks;
    protected $newCode;
    protected $nrorg;
    protected $operador;
    protected $currentTime;
    protected $crudService;
    protected $reportService;
    protected $fileUploader;
    
    ////////////////////////////////////////////////////////////////////////////
    
    //private $_tempEsocialPath;

    private $_xmlTemplatesPath;
    //private $_xsdTemplatesPath;
    private $_fileServerUrl;  
    private $_tempCertificatesPath;
    private $_tpInscTransmissor;
    private $_nrInscTransmissor;
    private $_certFile;
    private $_certPass;
    private $_certKeyFile;
    private $_xmlFilesPath;
    private $_arrXmlCaracter;
    private $_arrXmlEscape;
    private $_tpAmb;
    private $_procEmi;
    private $_verProc;
    private $_tipoOcorRetorno;
    private $_nmEvento;
    private $_idOperacao;
    private $_methodGetDados;
    private $_nrenvioTransmitir;
    private $_imposto;
    
    public function __construct(DataSourceManager $dataSourceManager, \Doctrine\ORM\EntityManager $entityManager, Environment $environment, NewCode $newCode ,Fis75000EfdReinfFactory $factory, Masks $masks, $fileServerUrl, ReportService $reportService, $fileUploader) {
        $this->dataSourceManager        = $dataSourceManager;
        $this->entityManager            = $entityManager;
        $this->environment              = $environment;
        $this->factory                  = $factory;
        $this->masks                    = $masks;
        $this->newCode                  = $newCode;
        $this->nrorg                    = $environment->getNrorgTrab();
        $this->operador                 = $this->environment->getCdOperador();
        $this->currentTime              = $this->environment->getCurrentTime()->format("d/m/Y H:i:s");
        $this->reportService            = $reportService;
        $this->fileUploader             = $fileUploader;
        
        $this->_projectsPath            = __DIR__ . DIRECTORY_SEPARATOR .'..'.DIRECTORY_SEPARATOR.'files'.DIRECTORY_SEPARATOR.'reinf'.DIRECTORY_SEPARATOR."datasource";
        $this->_fileServerUrl           = $fileServerUrl;
        $this->_tempCertificatesPath    = DIRECTORY_SEPARATOR.'tmp'. DIRECTORY_SEPARATOR . "certificates" . DIRECTORY_SEPARATOR . $this->nrorg;
        $this->_xmlFilesPath            = DIRECTORY_SEPARATOR.'tmp'. DIRECTORY_SEPARATOR . "xmlFiles" . DIRECTORY_SEPARATOR .$this->nrorg;
        $this->_xmlTemplatesPath        = __DIR__ . DIRECTORY_SEPARATOR .'..'.DIRECTORY_SEPARATOR.'files'.DIRECTORY_SEPARATOR.'reinf'.DIRECTORY_SEPARATOR."templates" . DIRECTORY_SEPARATOR . "xml";
        $this->_arrXmlCaracter          = array('>', '<', '&', '"', "'");
        $this->_arrXmlEscape            = array('&gt;', '&lt;', '&amp;', '&quot;', '&#39');        
        
        // Valor para produção e desenvolvimento
        $this->_verProc                 = '1.0.0';
        //Vaor para limpar base de produção restrita
        // $this->_verProc                 = 'RemoverContribuinte';
        
        $this->_procEmi                 = 1;
        
        $this->_tipoOcorRetorno = array(
            '1' => self::ERRO,
            '2' => self::ADVERTENCIA,
        );
        
        $this->_methodGetDados = 'getDados';
        
    }

    /******************************************************
    *                                                     *
    *    Funções pra validações                           *
    *                                                     *
    *                                                     *
    ** **************************************************/
    
    public function validaServicoDosItems($row, $nmEvento) {
        if ($nmEvento == self::R2010 || $nmEvento == self::R2020) {
            $evento = explode('-', $nmEvento);
            $data = DateUtil::getDataDeString(substr($row['DTMESCOMPETENC'], 0, 10));
            $ultimoDia = DateUtil::getUltimoDiaDoMes($data);
            $params = array(
                'CDPARTICIPANTE' => $row['CDESTABREINF'],
                'CDIMPOSTO' => $nmEvento == self::R2010 ? '41' : '42',
                'CDEMPRESA' => $row['CDEMPRESA'],
                'DTINICIAL' => '01/'.DateUtil::getMes($data).'/'.DateUtil::getAno($data),
                'DTFINAL' => DateUtil::getDia($ultimoDia).'/'.DateUtil::getMes($ultimoDia).'/'.DateUtil::getAno($ultimoDia)
            );
            $metodo = 'todasNotas_evento'.$evento[0].$evento[1];
            $notas = $this->factory->$metodo($params);
            $this->_imposto = $nmEvento == self::R2010 ? '41' : '42';
            $notasItemSemServico = array_filter($notas, $this->notaItemSemServico()); 
            return $notasItemSemServico;
        }  else {
            return array();
        }   
    }
    
    private function notaItemSemServico() {
        return function($nota) {
            $params = array(
                "NRLANCTONF" => $nota['NRLANCTONF'],
                "CDFILIAL" => $nota['CDFILIAL'],
                "CDIMPOSTO" => $this->_imposto,
                "CDEMPRESA"=> $nota['CDEMPRESA'],
                "CDINSCMUNI"=> $nota['CDINSCMUNI']
            );
            $itens = $this->factory->getItensDeNota($params);
            $itensSemServico = array_filter($itens, function($item) {
                return $item['CDCLASERVMOBEMP'];
            });
            
            return (count($itensSemServico) == 0);
        };
    }
    
    /******************************************************
    *                                                     *
    *    Funções Referentes a Manipulação e busca do XML  *
    *                                                     *
    *                                                     *
    ** **************************************************/
    
    public function getXmlLote($idGrupo, $empregador, $dsVersao, $eventosLote) {
        try {
            $xmlData = array(
                "IDGRUPO" => ($idGrupo == 'P' ? '3' : ($idGrupo == 'N' ? '2' : '1')),
                "EMPTPINSCRICAO" => $empregador['IDTPIJUREMPR'],
                "EMPNRINSCRICAO" => $empregador['NRINSJUREMPR'],
                "TRANSMTPINSCRICAO" => $this->_tpInscTransmissor,
                "TRANSMNRINSCRICAO" => $this->_nrInscTransmissor,
                "EVENTOS" => $eventosLote,
            );
            $nmevento = $this->_idOperacao != 'consulta' ? "envioLoteEventos" : "consultaLoteEventos";
            $xmlLote = $this->buildXml($nmevento, $dsVersao, $xmlData, $nmevento, true);
            return $xmlLote;
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    private function buildXml($nmEvento, $dsVersao, $data, $nameToSave = '', $saveXml=false) {
        try {
            //$dsVersao  = str_replace('.','-',$dsVersao);
            $templateDir = $this->_xmlTemplatesPath . DIRECTORY_SEPARATOR."v_$dsVersao";
            $templateBuilder = new Template($data, $templateDir);
            $xml = $templateBuilder->build("{$nmEvento}.xml.twig");
            if (!is_null($nameToSave) && $saveXml) {
                $fileName = $nameToSave . ".xml";
                $fileDir = $this->_xmlFilesPath;
                if (!is_dir($fileDir)) {
                    mkdir($fileDir, 0777, true);
                }
                if (is_file($fileDir . DIRECTORY_SEPARATOR . $fileName)) {
                    unlink($fileDir . DIRECTORY_SEPARATOR . $fileName);
                }
                file_put_contents($fileDir . DIRECTORY_SEPARATOR . $fileName, $this->formatXML($xml));
                chmod($fileDir . DIRECTORY_SEPARATOR . $fileName, 0777);
            }
            return $xml;
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    private function formatXML($xml){
        // Substitui caracteres especias que devem ser evitados ao enviar o xml para o eSocial
        $xml = str_replace($this->_arrXmlEscape, $this->_arrXmlCaracter, $xml);

        // Este bloco transforma o texto gerado, para que as tags do twig sejam ocultadas, a fim de possibilitar a
        // formatacao do documentos, que somente e disponivel para um texto no formato .xml
        $xml = str_replace("{{", "aaa{{", $xml);
        $xml = str_replace("}}", "}}aaa", $xml);
        $xml = str_replace("{%", "<TEMPORARY_TAG>aaa{%", $xml);
        $xml = str_replace("%}", "%}aaa</TEMPORARY_TAG>", $xml);

        $xml = str_replace("&lt;", "<", $xml);
        $xml = str_replace("&gt;", ">", $xml);
        $xml = str_replace("&quot;", '"', $xml);
        $xml = str_replace("&", "e", $xml);     // Retirado carater '&' pois estava dando erro quando existia em algum campo de descrição

        $xml    = new \SimpleXMLElement($xml);
        $domxml = new \DOMDocument();
        $domxml->preserveWhiteSpace = false;
        $domxml->formatOutput = true;
        $domxml->loadXML($xml->asXML());

        // Remocao do texto temporario gerado.
        //$xml =  $domxml->saveHTML();
        $xml =  $domxml->saveXML();
        $xml = str_replace("aaa{{", "{{", $xml);
        $xml = str_replace("}}aaa", "}}", $xml);
        $xml = str_replace("<TEMPORARY_TAG>aaa{%", "{%", $xml);
        $xml = str_replace("%}aaa</TEMPORARY_TAG>", "%}", $xml);

        // Remove a tag XML "<?xml version='1.0'>" pois não é utilizada ao montar o SOAP para envio
        $xml = preg_replace('!^[^>]+>(\r\n|\n)!','',$xml);

        return $xml;
    }
    
    public function getXmlEvento($dsVersao, $dadosEnvio) {
        try {
            /* Monta os dados do evento para gerar o XML */
            $xmlData = $this->buildDadosXml($dsVersao, $dadosEnvio, $this->_tpAmb, $this->_procEmi, $this->_verProc);
        //    var_dump('oii');
            if(isset($dadosEnvio)){
                if($dadosEnvio["NMEVENTO"] == "R-1000"){
                    $CDNATUJURI = $this->factory->getCDNATUJURI($this->nrorg, $this->_cdempresa);
                    $CDNATUJURI = $CDNATUJURI["CDNATUJURI"];
                    
                    if(isset($CDNATUJURI) || is_null($CDNATUJURI)){    
                        if(!(($CDNATUJURI == '1023') ||
                            ($CDNATUJURI == '1031') ||
                            ($CDNATUJURI == '1058') ||
                            ($CDNATUJURI == '1066') ||
                            ($CDNATUJURI == '1082') ||
                            ($CDNATUJURI == '1171') ||
                            ($CDNATUJURI == '1180') ||
                            ($CDNATUJURI == '1236') ||
                            ($CDNATUJURI == '1244'))){
                                unset($xmlData["XMLDATA"]["evtInfoContri"]["infoContri"]["inclusao"]["infoCadastro"]["infoEFR"]);
                        }
                    }
                }
            }
           // var_dump($xmlData);die;
            if ($xmlData) {
                /* Utiliza os dados formatados para montar o xml conforme o template twig */
                $nmXmlEvento = 'envio_'.$xmlData['IDEVENTO'];
                $xmlEvento = $this->buildXml($dadosEnvio['NMEVENTO'], $dsVersao, $xmlData['XMLDATA'], $nmXmlEvento);
                /* Assina o xml do evento */
                $xmlEvento = $this->assinaXmlEvento($xmlEvento, $dadosEnvio);
                if (!is_null($dadosEnvio['DSDIRXMLENV'])) {
                    $this->fileUploader->deleteFile($dadosEnvio['DSDIRXMLENV']);
                }
                

                //OLHAR SEGUNDAAAAAAAA
                /*
                $xmlPath = $this->saveXmlOnFileServer(
                    $dadosEnvio['NRESTRUTURAM'], null, $xmlEvento, $nmXmlEvento, $xmlData['IDEVENTO']
                );*/
                
                /*
                $envio = $this->entityManager->getRepository(Repositories::ESO_ENVIOCADASTROS)->find(array(
                    'nrenvio' => $dadosEnvio['NRENVIO'], 'nrorg' => $this->hcmParameters->getNrorgTrabalho()
                ));
                $envio->setIdevento($xmlData['IDEVENTO']);
                $envio->setDtenvio(DateUtil::getDataAtual());
                $envio->setDsdirxmlenv($xmlPath);
                $envio->setDtultatu(DateUtil::getDataAtual());
                $envio->setCdoperultatu($this->hcmParameters->getCdoperador());
                $envio->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
                */
                

                return array(
                    "DSVERSAO" =>$dadosEnvio['DSVERSAO'],
                    "NMEVENTO"=> $dadosEnvio['NMEVENTO'],
                    "NRENVIO" => $dadosEnvio['NRENVIO'],
                    "IDEVENTO" => $xmlData['IDEVENTO'],
                    "NMXMLEVENTO" => $nmXmlEvento,
                    "XMLEVENTO" => $xmlEvento,
                );
            } else {
                return false;
            }
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function getXmlEventoLimparBase($dsVersao, $dadosEnvio) {
        try {
            /* Monta os dados do evento para gerar o XML */
            $xmlData = $this->buildDadosXmlLimparLote($dsVersao, $dadosEnvio, $dadosEnvio['TPAMB'], $dadosEnvio['PROCEMI'], $dadosEnvio['VERPROC']);
            
            if(isset($dadosEnvio)){
                if($dadosEnvio["NMEVENTO"] == "R-1000"){
                    $CDNATUJURI = $this->factory->getCDNATUJURI($this->nrorg, $this->_cdempresa);
                    $CDNATUJURI = $CDNATUJURI["CDNATUJURI"];
                    
                    if(isset($CDNATUJURI) || is_null($CDNATUJURI)){    
                        if(!(($CDNATUJURI == '1023') ||
                            ($CDNATUJURI == '1031') ||
                            ($CDNATUJURI == '1058') ||
                            ($CDNATUJURI == '1066') ||
                            ($CDNATUJURI == '1082') ||
                            ($CDNATUJURI == '1171') ||
                            ($CDNATUJURI == '1180') ||
                            ($CDNATUJURI == '1236') ||
                            ($CDNATUJURI == '1244'))){
                                unset($xmlData["XMLDATA"]["evtInfoContri"]["infoContri"]["inclusao"]["infoCadastro"]["infoEFR"]);
                        }
                    }
                }
            }
            
            if ($xmlData) {
                /* Utiliza os dados formatados para montar o xml conforme o template twig */
                $nmXmlEvento = 'envio_'.$xmlData['IDEVENTO'];
                $xmlEvento = $this->buildXml($dadosEnvio['NMEVENTO'], $dsVersao, $xmlData['XMLDATA'], $nmXmlEvento);
                /* Assina o xml do evento */
                $xmlEvento = $this->assinaXmlEvento($xmlEvento, $dadosEnvio);
                if (!is_null($dadosEnvio['DSDIRXMLENV'])) {
                    $this->fileUploader->deleteFile($dadosEnvio['DSDIRXMLENV']);
                }
                
                /*
                $xmlPath = $this->saveXmlOnFileServer(
                    $dadosEnvio['NRESTRUTURAM'], null, $xmlEvento, $nmXmlEvento, $xmlData['IDEVENTO']
                );*/
                
                /*
                $envio = $this->entityManager->getRepository(Repositories::ESO_ENVIOCADASTROS)->find(array(
                    'nrenvio' => $dadosEnvio['NRENVIO'], 'nrorg' => $this->hcmParameters->getNrorgTrabalho()
                ));
                $envio->setIdevento($xmlData['IDEVENTO']);
                $envio->setDtenvio(DateUtil::getDataAtual());
                $envio->setDsdirxmlenv($xmlPath);
                $envio->setDtultatu(DateUtil::getDataAtual());
                $envio->setCdoperultatu($this->hcmParameters->getCdoperador());
                $envio->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
                */
                

                return array(
                    "DSVERSAO" =>$dadosEnvio['DSVERSAO'],
                    "NMEVENTO"=> $dadosEnvio['NMEVENTO'],
                    "NRENVIO" => $dadosEnvio['NRENVIO'],
                    "IDEVENTO" => $xmlData['IDEVENTO'],
                    "NMXMLEVENTO" => $nmXmlEvento,
                    "XMLEVENTO" => $xmlEvento,
                );
            } else {
                return false;
            }
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function buildDadosXml($dsVersao, $dadosEnvio, $tpAmb, $procEmi, $verProc) {
        try {
            $projects = json_decode(file_get_contents($this->_projectsPath . "/projects_v_$dsVersao.json"), true);
            //$helpers = json_decode(file_get_contents($this->_helpersPath . "/helper_v_$dsVersao.json"), true);
            
            $idEvento = null;
            $dadosXml = null;
           // var_dump($projects);die;
            foreach ($projects as $nomeEvento => $agrupadores) {
                if ($nomeEvento == $dadosEnvio['NMEVENTO']) {
                    $this->_nmEvento = str_replace('-','',$nomeEvento);
                    $this->_idOperacao = $this->getOperacaoEvento($dadosEnvio['IDOPERACAO']);
                    // Método dinâmico para buscar os dados do evento (Ex: getDadosS1000)
                    $method = $this->_methodGetDados . $this->_nmEvento;
                    
                    if (method_exists($this, $method)) {
                        $dados = $this->$method($dadosEnvio);
                     //   var_dump($dadosEnvio);die;
                        if ($dados) {
                            // Utiliza o ID gerado pela query caso o envio ainda não tenha
                            $idEvento = is_null($dadosEnvio['IDEVENTO']) ? $dados['ID'] : $dadosEnvio['IDEVENTO'];
                           
                            /* Seta as variáveis de ambiente */
                            $dados['TPAMB']    = $dadosEnvio['IDAMBIENTE']; //$tpAmb;
                            $dados['PROCEMI']  = $procEmi;
                            $dados['VERPROC']  = $verProc;
                          //  var_dump($agrupadores);die;
                            /* Formata os dados do evento conforme o objeto de descrição json*/
                            $dadosXml = $this->percorreAgrupadores($agrupadores, $dados, $dadosEnvio);
                            return array(
                                'IDEVENTO' => $idEvento,
                                'XMLDATA' => $dadosXml
                            );
                        }
                    } else {
                        return false;
                    }
                }
            }
            return false;
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function buildDadosXmlLimparLote($dsVersao, $dadosEnvio, $tpAmb, $procEmi, $verProc) {
        try {
            $projects = json_decode(file_get_contents($this->_projectsPath . "/projects_v_$dsVersao.json"), true);
            //$helpers = json_decode(file_get_contents($this->_helpersPath . "/helper_v_$dsVersao.json"), true);
            
            $idEvento = null;
            $dadosXml = null;
            foreach ($projects as $nomeEvento => $agrupadores) {
                if ($nomeEvento == $dadosEnvio['NMEVENTO']) {
                    $this->_nmEvento = str_replace('-','',$nomeEvento);
                    $this->_idOperacao = $this->getOperacaoEvento($dadosEnvio['IDOPERACAO']);
                    // Método dinâmico para buscar os dados do evento (Ex: getDadosS1000)
                    $dados = $this->getDadosR1000LimparBase($dadosEnvio['CDEMPRESA']);
                    if ($dados) {
                        // Utiliza o ID gerado pela query caso o envio ainda não tenha
                        $idEvento = is_null($dadosEnvio['IDEVENTO']) ? $dados['ID'] : $dadosEnvio['IDEVENTO'];
                       
                        /* Seta as variáveis de ambiente */
                        $dados['TPAMB']    = $dadosEnvio['IDAMBIENTE']; //$tpAmb;
                        $dados['PROCEMI']  = $procEmi;
                        $dados['VERPROC']  = $verProc;
                        
                        /* Formata os dados do evento conforme o objeto de descrição json*/
                        $dadosXml = $this->percorreAgrupadores($agrupadores, $dados, $dadosEnvio);
                        return array(
                            'IDEVENTO' => $idEvento,
                            'XMLDATA' => $dadosXml
                        );
                    }
                }
            }
            return false;
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    private function percorreAgrupadores($agrupadores, $dadosAgrupador, $dadosEnvio) {
        $dadosXml = null;
        
        foreach ($agrupadores as $nmAgrupador => $vrAgrupador) {
            
            if (is_array($vrAgrupador)) {
                if (isset($vrAgrupador[0])) {
                    
                    // Para os itens de grupo que podem se repetir mais de uma vez
                    // Se for array de array busca os dados para preencher -> Ocorrencia maior que 1 no layout do eSocial
                    $dadosXml[$nmAgrupador] = array();

                    // Método dinâmico para buscar os dados de parte do evento (Ex: getDadosS1000softwareHouse)
                    $method = $this->_methodGetDados . $this->_nmEvento . $nmAgrupador;
                    
                    if (method_exists($this, $method)) {
                        if ($nmAgrupador == 'infoTpServ') {
                            $dadosAux = $this->$method($dadosAgrupador, $dadosEnvio, $dadosAgrupador);
                        } else {
                            $dadosAux = $this->$method($dadosAgrupador, $dadosEnvio);
                        }
                    } else {
                        $dadosAux = array();
                    }
                    
                    // Percorre os dados retornandos setando na propriedade do evento
                    foreach ($dadosAux as $dados) {
                        array_push($dadosXml[$nmAgrupador],$this->percorreAgrupadores($vrAgrupador[0], $dados, $dadosEnvio));
                    }
                } else {
                    // Caso o evento possua inclusão/alteração/exclusão verifica o tipo do envio a ser gerado
                    if (($nmAgrupador == 'inclusao' || $nmAgrupador == 'alteracao' || $nmAgrupador == 'exclusao') && ($nmAgrupador != $this->_idOperacao)) {
                        continue;
                    }
                    // Para os itens de grupo que não se repetem
                    $dadosXml[$nmAgrupador] = $this->percorreAgrupadores($vrAgrupador, $dadosAgrupador, $dadosEnvio);
                    if ($this->agrupadorIsEmpty($dadosXml[$nmAgrupador])) {
                        unset($dadosXml[$nmAgrupador]);
                    }
                }
            } else {
                // Busca o valor no array de dados para os itens do tipo atributo e elemento
                $dadosXml[strtoupper($nmAgrupador)] = $this->getValorAgrupador($dadosAgrupador, $vrAgrupador);
                // Executas regras do campo
                /*
                if (isset($regras[strtoupper($nmAgrupador)])) {
                    $this->executaRegras(strtoupper($nmAgrupador), $regras[strtoupper($nmAgrupador)], $dadosXml);
                }
                */
            }
        }
        //Remover campos vazios  no array para não enviar tags vazias no xml
        $dadosXml = array_filter($dadosXml, function($value) {
            return ! is_null($value);
        });
        
        return $dadosXml;
    }
    
    /**
     * Retorna o valor do agrupador conforme o array de dados
     */
    private function getValorAgrupador($dadosAgrupador, $vrAgrupador) {
        foreach ($dadosAgrupador as $name => $value) {
            if ($name == $vrAgrupador) {
                return $value;
            }
        }
        return null;
    }
    
    /**
     * Verifica se o agrupador está vazio
     */
    private function agrupadorIsEmpty($agrupador) {
        foreach ($agrupador as $index => $value) {
            if ((is_array($value) and !empty($value)) || (!is_array($value) && !is_null($value) && $value != '')) {
                return false;
            }
        }
        return true;
    }
    
    public function geraXmlConsulta($nrenvio) {
        $dadosConsulta = $this->factory->getDadosConsulta($nrenvio);
        
        $xml = '<soapenv:Envelope xmlns:soapenv="http://schemas.xmlsoap.org/soap/envelope/" xmlns:sped="http://sped.fazenda.gov.br/">
                   <soapenv:Header/>
                   <soapenv:Body>
                      <sped:ConsultaResultadoFechamento2099>
                         <sped:tipoInscricaoContribuinte>'.$dadosConsulta['FISICOJURIDICO'].'</sped:tipoInscricaoContribuinte>
                         <sped:numeroInscricaoContribuinte>'.$dadosConsulta['CODIGO'].'</sped:numeroInscricaoContribuinte>
                         <sped:numeroProtocoloFechamento>'.$dadosConsulta['NRRECIBO'].'</sped:numeroProtocoloFechamento>
                      </sped:ConsultaResultadoFechamento2099>
                   </soapenv:Body>
                </soapenv:Envelope>';
        
        return $xml;
    }
    
    public function geraXmlConsulta1000($row) {
        
        $dadosConsulta  = $this->factory->getDadosConsulta($row['NRENVIO']);
        
        $xml = '<soapenv:Envelope xmlns:soapenv="http://schemas.xmlsoap.org/soap/envelope/" xmlns:sped="http://sped.fazenda.gov.br/">
                   <soapenv:Header/>
                   <soapenv:Body>
                      <sped:ConsultaReciboEvento1000>
                         <sped:tipoEvento>'         .'1000'.                            '</sped:tipoEvento>
                         <sped:tpInsc>'             .$dadosConsulta['FISICOJURIDICO'].  '</sped:tpInsc>
                         <sped:nrInsc>'             .$dadosConsulta['CODIGO'].          '</sped:nrInsc>
                      </sped:ConsultaReciboEvento1000>
                   </soapenv:Body>
                </soapenv:Envelope>';
        
        return $xml;
    }
    
    public function geraXmlConsulta2010($row) {
        
        $data = DateUtil::getDataDeString(substr($row['DTMESCOMPETENC'], 0, 10));
        $ultimoDia = DateUtil::getUltimoDiaDoMes($data);
        $dtinicial = '01/'.DateUtil::getMes($data).'/'.DateUtil::getAno($data);
        $dtfinal = DateUtil::getDia($ultimoDia).'/'.DateUtil::getMes($ultimoDia).'/'.DateUtil::getAno($ultimoDia);
        
        $mapaApurac     = $this->factory->getMapaPeriodo($row['CDEMPRESA'], $dtinicial, $dtfinal, '41');
        $dadosConsulta  = $this->factory->getDadosConsulta2010($row['CDEMPRESA'], $dtinicial, $row['NRENVIO'], $mapaApurac['NRSEQUMAPAPU']);
        
        $xml = '<soapenv:Envelope xmlns:soapenv="http://schemas.xmlsoap.org/soap/envelope/" xmlns:sped="http://sped.fazenda.gov.br/">
                   <soapenv:Header/>
                   <soapenv:Body>
                      <sped:ConsultaReciboEvento2010>
                         <sped:tipoEvento>'         .'2010'.                            '</sped:tipoEvento>
                         <sped:tpInsc>'             .$dadosConsulta['TPINSC'].          '</sped:tpInsc>
                         <sped:nrInsc>'             .$dadosConsulta['NRINSC'].          '</sped:nrInsc>
                         <sped:perApur>'            .$dadosConsulta['PERAPUR'].         '</sped:perApur>
                         <sped:tpInscEstab>'        .$dadosConsulta['TPINSCESTAB'].     '</sped:tpInscEstab>
                         <sped:nrInscEstab>'        .$dadosConsulta['NRINSCESTAB'].     '</sped:nrInscEstab>
                         <sped:cnpjPrestador>'      .$dadosConsulta['CNPJPRESTADOR'].   '</sped:cnpjPrestador>
                      </sped:ConsultaReciboEvento2010>
                   </soapenv:Body>
                </soapenv:Envelope>';
        
        return $xml;
    }
    
    public function geraXmlConsulta2020($row) {
        
        $data = DateUtil::getDataDeString(substr($row['DTMESCOMPETENC'], 0, 10));
        $ultimoDia = DateUtil::getUltimoDiaDoMes($data);
        $dtinicial = '01/'.DateUtil::getMes($data).'/'.DateUtil::getAno($data);
        $dtfinal = DateUtil::getDia($ultimoDia).'/'.DateUtil::getMes($ultimoDia).'/'.DateUtil::getAno($ultimoDia);
        
        $mapaApurac     = $this->factory->getMapaPeriodo($row['CDEMPRESA'], $dtinicial, $dtfinal, '42');
        $dadosConsulta  = $this->factory->getDadosConsulta2020($row['CDEMPRESA'], $dtinicial, $row['NRENVIO'], $mapaApurac['NRSEQUMAPAPU']);
        
        $xml = '<soapenv:Envelope xmlns:soapenv="http://schemas.xmlsoap.org/soap/envelope/" xmlns:sped="http://sped.fazenda.gov.br/">
                   <soapenv:Header/>
                   <soapenv:Body>
                      <sped:ConsultaReciboEvento2020>
                         <sped:tipoEvento>'         .'2020'.                            '</sped:tipoEvento>
                         <sped:tpInsc>'             .$dadosConsulta['TPINSC'].          '</sped:tpInsc>
                         <sped:nrInsc>'             .$dadosConsulta['NRINSC'].          '</sped:nrInsc>
                         <sped:perApur>'            .$dadosConsulta['PERAPUR'].         '</sped:perApur>
                         <sped:nrInscEstabPrest>'   .$dadosConsulta['NRINSCESTABPREST'].'</sped:nrInscEstabPrest>
                         <sped:tpInscTomador>'      .$dadosConsulta['TPINSCTOMADOR'].   '</sped:tpInscTomador>
                         <sped:nrInscTomador>'      .$dadosConsulta['NRINSCTOMADOR'].   '</sped:nrInscTomador>
                      </sped:ConsultaReciboEvento2020>
                   </soapenv:Body>
                </soapenv:Envelope>';
        
        return $xml;
    }
    
    public function geraXmlConsulta2060($row) {
        
        $data = DateUtil::getDataDeString(substr($row['DTMESCOMPETENC'], 0, 10));
        $ultimoDia = DateUtil::getUltimoDiaDoMes($data);
        $dtinicial = '01/'.DateUtil::getMes($data).'/'.DateUtil::getAno($data);
        $dtfinal = DateUtil::getDia($ultimoDia).'/'.DateUtil::getMes($ultimoDia).'/'.DateUtil::getAno($ultimoDia);
        
        $dadosConsulta  = $this->factory->getDadosConsulta2060($row['CDEMPRESA'], $dtinicial, $dtfinal);
        
        $xml = '<soapenv:Envelope xmlns:soapenv="http://schemas.xmlsoap.org/soap/envelope/" xmlns:sped="http://sped.fazenda.gov.br/">
                   <soapenv:Header/>
                   <soapenv:Body>
                      <sped:ConsultaReciboEvento2060>
                         <sped:tipoEvento>'         .'2060'.                            '</sped:tipoEvento>
                         <sped:tpInsc>'             .$dadosConsulta['TPINSC'].          '</sped:tpInsc>
                         <sped:nrInsc>'             .$dadosConsulta['NRINSC'].          '</sped:nrInsc>
                         <sped:perApur>'            .$dadosConsulta['PERAPUR'].         '</sped:perApur>
                         <sped:tpInscEstab>'        .$dadosConsulta['TPINSCESTAB'].     '</sped:tpInscEstab>
                         <sped:nrInscEstab>'        .$dadosConsulta['NRINSCESTAB'].     '</sped:nrInscEstab>
                      </sped:ConsultaReciboEvento2060>
                   </soapenv:Body>
                </soapenv:Envelope>';
        
        return $xml;
    }
    
    public function geraXmlConsulta2098($row) {
        
        $dadosConsulta  = $this->factory->getDadosConsulta2098($row['CDEMPRESA'], $row['DTMESCOMPETENC']);
        
        $xml = '<soapenv:Envelope xmlns:soapenv="http://schemas.xmlsoap.org/soap/envelope/" xmlns:sped="http://sped.fazenda.gov.br/">
                   <soapenv:Header/>
                   <soapenv:Body>
                      <sped:ConsultaReciboEvento2098>
                         <sped:tipoEvento>'         .'2098'.                            '</sped:tipoEvento>
                         <sped:tpInsc>'             .$dadosConsulta['TPINSC'].          '</sped:tpInsc>
                         <sped:nrInsc>'             .$dadosConsulta['NRINSC'].          '</sped:nrInsc>
                         <sped:perApur>'            .$dadosConsulta['PERAPUR'].         '</sped:perApur>
                      </sped:ConsultaReciboEvento2098>
                   </soapenv:Body>
                </soapenv:Envelope>';
        
        return $xml;
    }
    
    public function geraXmlConsulta2099($row) {
        
        $dadosConsulta  = $this->factory->getDadosConsulta2098($row['CDEMPRESA'], $row['DTMESCOMPETENC']);
        
        $xml = '<soapenv:Envelope xmlns:soapenv="http://schemas.xmlsoap.org/soap/envelope/" xmlns:sped="http://sped.fazenda.gov.br/">
                   <soapenv:Header/>
                   <soapenv:Body>
                      <sped:ConsultaReciboEvento2099>
                         <sped:tipoEvento>'         .'2099'.                            '</sped:tipoEvento>
                         <sped:tpInsc>'             .$dadosConsulta['TPINSC'].          '</sped:tpInsc>
                         <sped:nrInsc>'             .$dadosConsulta['NRINSC'].          '</sped:nrInsc>
                         <sped:perApur>'            .$dadosConsulta['PERAPUR'].         '</sped:perApur>
                      </sped:ConsultaReciboEvento2099>
                   </soapenv:Body>
                </soapenv:Envelope>';
        
        return $xml;
    }
    
    /******************************************************
    *                                                     *
    *              Assinatura XML Eventos                 *
    *                                                     *
    *                                                     *
    ******************************************************/
    
    public function assinaXmlEvento($xmlEvento, $dadosEnvio) {
        try {
            $xmlEvento = utf8_encode($xmlEvento);
            /* Formata o xml*/
            $domXml = new \DOMDocument("1.0", "UTF-8");
            $domXml->loadXML($xmlEvento, LIBXML_NOBLANKS);
            //$domXml = $this->removerTagsVazias($domXml); //opcional
            $domXml->formatOutput = true;
            $xmlTemp = $domXml->saveXML($domXml->documentElement);
           
            $domXml = new \DOMDocument("1.0", "UTF-8");
            $domXml->loadXML($xmlTemp);
            $domXml->encoding = "UTF-8";

            /* Busca qual tag será assinada */
            $tagAssinatura = "";
            $id = "";
            /** @var \DOMNode $nodeEvento */
            $nodeEvento = $domXml->getElementsByTagName('Reinf')->item(0)->childNodes[1]; //Pega o nome da tag principal do evento
            //$id = $nodeEvento->getAttribute("Id");
            $tagAssinatura = $nodeEvento->nodeName;
            /** @var \DOMNode $node */
            /*foreach ($nodeEvento->childNodes as $node) {
                if ($node->nodeType == 1 && ($node->nodeName != 'ideEvento' && $node->nodeName != 'ideEmpregador')) {
                    $tagAssinatura = $node->nodeName;
                    $node->setAttribute("Id", $id);
                }
            }*/
            $domXmlAssinado = $this->assinaXml($domXml, array($tagAssinatura), $dadosEnvio);
            $xmlAssinado = $domXmlAssinado->saveXML($domXmlAssinado->documentElement);
            return $xmlAssinado;
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    private function assinaXml(\DOMDocument $domXml, array $tags, $dadosEnvio) {
        try {
            //if (is_dir($this->_fileServerUrl)) {
            //if (is_file($this->_certFile)) {
             
            if (fopen($this->_certFile,"r") == true) {
                // Get file name without extension
               // $data = file_get_contents($this->_certFile);
                $dados = $this->factory->buscardados($dadosEnvio);
                $dados['DSDIRCERTIFICADO'] = explode('>',$dados['DSDIRCERTIFICADO'])[1] ; 
                        //var_dump($link);die;
                 $link = $this->_fileServerUrl.$dados['DSDIRCERTIFICADO'];
                 $data = file_get_contents($link);
                openssl_pkcs12_read($data, $cert, $this->_certPass);

                if (is_null($cert)) {

                    throw new \Exception("Certificado inválido ou senha inválida");
                } else {
                    file_put_contents($this->_tempCertificatesPath . DIRECTORY_SEPARATOR . "certKey". $this->_nrInscTransmissor . ".pem", $cert["pkey"] . "\r\n" . $cert["cert"]);
                    chmod($this->_tempCertificatesPath . DIRECTORY_SEPARATOR . "certKey". $this->_nrInscTransmissor . ".pem", 0777);

                    $chavePublica = $this->getPublicKey($cert["cert"]);
                    $chavePrivada = openssl_get_privatekey($cert["pkey"]);

                    foreach ($tags as $tag) {
                        $domXml = $this->assinaXmlAux($domXml, $tag, $chavePublica, $chavePrivada);
                    }

                    openssl_free_key($chavePrivada);
                    return $domXml;
                }
            } else {
                throw new \Exception(MessagesFiscal::CERT_NAO_ECONTRADO,1);
            }
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    private function getPublicKey($key) {
        $data = "";
        $arCert = explode("\n", $key);
        foreach ($arCert as $curData) {
            //remove a tag de inicio e fim do certificado
            if (strncmp($curData, "-----BEGIN CERTIFICATE", 22) != 0 &&
                strncmp($curData, "-----END CERTIFICATE", 20) != 0 ) {
                //carrega o resultado numa string
                $data .= trim($curData);
            }
        }
        return $data;
    }

    private function assinaXmlAux(\DOMDocument $domXml, $tag, $chavePublica, $chavePrivada) {
        try {
            $nodes = $domXml->getElementsByTagName($tag);
            foreach ($nodes as $node) {
                $root = $node->parentNode;
                $id = trim($node->getAttribute("id"));
                $dados = $node->C14N(false, false, null, null);
                //$dados = $root->C14N(false, false, null, null);
                
                //$hashValue = hash("sha1", $dados, true);
                $hashValue = hash("sha256", $dados, true);
                $digValue = base64_encode($hashValue);

                $signature = $domXml->createElement("Signature");
                $root->appendChild($signature);
                $signature->setAttribute("xmlns", "http://www.w3.org/2000/09/xmldsig#");
                //$signature->setAttribute("Id", "Ass_" . $id);

                $signedInfo = $domXml->createElement("SignedInfo");
                $signature->appendChild($signedInfo);

                $novaTag = $domXml->createElement("CanonicalizationMethod");
                $signedInfo->appendChild($novaTag);
                $novaTag->setAttribute("Algorithm", "http://www.w3.org/TR/2001/REC-xml-c14n-20010315");

                $novaTag = $domXml->createElement("SignatureMethod");
                $signedInfo->appendChild($novaTag);
                //$novaTag->setAttribute("Algorithm", "http://www.w3.org/2000/09/xmldsig#rsa-sha1");
                $novaTag->setAttribute("Algorithm", "http://www.w3.org/2001/04/xmldsig-more#rsa-sha256");

                $reference = $domXml->createElement("Reference");
                $signedInfo->appendChild($reference);
                $reference->setAttribute("URI", "#" . $id);
                //$reference->setAttribute("URI", "");

                $transforms = $domXml->createElement("Transforms");
                $reference->appendChild($transforms);

                $novaTag = $domXml->createElement("Transform");
                $transforms->appendChild($novaTag);
                $novaTag->setAttribute("Algorithm", "http://www.w3.org/2000/09/xmldsig#enveloped-signature");

                $novaTag = $domXml->createElement("Transform");
                $transforms->appendChild($novaTag);
                $novaTag->setAttribute("Algorithm", "http://www.w3.org/TR/2001/REC-xml-c14n-20010315");

                $novaTag = $domXml->createElement("DigestMethod");
                $reference->appendChild($novaTag);
                //$novaTag->setAttribute("Algorithm", "http://www.w3.org/2000/09/xmldsig#sha1");
                $novaTag->setAttribute("Algorithm", "http://www.w3.org/2001/04/xmlenc#sha256");

                $novaTag = $domXml->createElement("DigestValue", $digValue);
                $reference->appendChild($novaTag);

                $dados = $signedInfo->C14N(false, false, null, null);
                $assinatura = "";
                openssl_sign($dados, $assinatura, $chavePrivada, "SHA256");
                $signatureValue = base64_encode($assinatura);


                $novaTag = $domXml->createElement("SignatureValue", $signatureValue);
                $signature->appendChild($novaTag);

                $keyInfo = $domXml->createElement("KeyInfo");
                $signature->appendChild($keyInfo);

                $X509Data = $domXml->createElement("X509Data");
                $keyInfo->appendChild($X509Data);

                $novaTag = $domXml->createElement("X509Certificate", $chavePublica);
                $X509Data->appendChild($novaTag);
            }
            return $domXml;
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    
    /***************************************************
    *                                                  *
    * Funções Referentes ao envio e a Comunicação  e   *
    *                     ao Retorno                   *
    *                                                  *
    *                                                  *
    *                                                  *
    ***************************************************/
    
    public function setDadosTransmissao($row) {
        
        try {
            $parametro = $this->getParamReinf($row['CDEMPRESA'])[0];
            $this->_tpAmb             = $parametro['IDAMBIENTE'];
            $this->_tpInscTransmissor = $row['IDTPIJUREMPR'];
            $this->_nrInscTransmissor = $row['NRINSJUREMPR'];
            $this->_cdempresa         = $row['CDEMPRESA'];
            $this->_nrenvioTransmitir = $row['NRENVIO'];
            $this->_idOperacao = $this->getOperacaoEvento($row['IDOPERACAO']);
            //$parametro['DSDIRCERTIFICADO'] = "<Certificado A1 Indalabor 2020.pfx>tkOrg15cb89fa736f6e/Certificados/60e601fad3862.pfx";
           // var_dump( $parametro['DSDIRCERTIFICADO']);die;
            if ($parametro) {
                if (empty($parametro['DSDIRCERTIFICADO'])) {
                    throw new \Exception("Certificado do transmissor não encontrado. Favor carregar o certificado na tela 'Parametros do Reinf'.");
                }
                if (empty($parametro['CDSENHACERTDIG'])) {
                    throw new \Exception("Senha do certificado do transmissor não encontrada. Favor informar a senha na tela 'Parametros do Reinf'.");
                }
           
                $parametro['CDSENHACERTDIG'] = Utilities::zeedhiDecrypt($parametro['CDSENHACERTDIG']);
                //$parametro['CDSENHACERTDIG']   = $this->decrypt($parametro['CDSENHACERTDIG'] );
            //    var_dump($parametro['CDSENHACERTDIG']);die;
                $parametro['DSDIRCERTIFICADO'] = explode('>',$parametro['DSDIRCERTIFICADO'])[1] ;   
                
                //$this->_certFile    = __DIR__.DIRECTORY_SEPARATOR.'CERTIFICADO SAPORE 2018.pfx';
                $this->_certFile    = "https://google.com";
                
                $this->_certKeyFile = $this->_tempCertificatesPath . DIRECTORY_SEPARATOR . "certKey" . $this->_nrInscTransmissor . ".pem";
                $this->_certPass    = $parametro['CDSENHACERTDIG'];
               // var_dump($parametro);die;
                if (fopen($this->_certFile,"r") == true) {
                   // var_dump($this->_certFile,"r");die;
                    if (!file_exists($this->_certKeyFile)) {
                        if (!is_dir($this->_tempCertificatesPath)) {
                            mkdir($this->_tempCertificatesPath, 0777, true);
                        }
                       
                        $link = $this->_fileServerUrl.$parametro['DSDIRCERTIFICADO'];
                        $data = file_get_contents($link);
                        openssl_pkcs12_read($data, $cert, $this->_certPass);
                        
                        if (is_null($cert)) {
                            throw new \Exception("Certificado inválido ou senha inválida");
                        } else {
                            $detail = openssl_x509_parse($cert["cert"], false);
                        //    var_dump(isset($detail["validTo"]));die;
                            if(isset($detail["validTo"])) {
                                if(strtotime(\DateTime::createFromFormat('ymdHis\Z', $detail['validTo'])->format("m/d/Y")) < strtotime(date("m/d/Y"))){ 
                                    throw new \Exception("Certificado expirado.");
                                }
                            }
                            file_put_contents($this->_tempCertificatesPath . DIRECTORY_SEPARATOR . "certKey" . $this->_nrInscTransmissor . ".pem", $cert["pkey"] . "\r\n" . $cert["cert"]);
                            chmod($this->_tempCertificatesPath . DIRECTORY_SEPARATOR . "certKey" . $this->_nrInscTransmissor . ".pem", 0777);
                        }
                    
                    } else {
                        $link = $this->_fileServerUrl.$parametro['DSDIRCERTIFICADO'];
                         $data = file_get_contents($link);
                        openssl_pkcs12_read($data, $cert, $this->_certPass);
                       /// var_dump($cert);die;
                        if (is_null($cert)) {
                            throw new \Exception("Certificado inválido ou senha inválida");
                        } else {
                            $detail = openssl_x509_parse($cert["cert"], false);
                            if(isset($detail["validTo"])) {
                                if(strtotime(\DateTime::createFromFormat('ymdHis\Z', $detail['validTo'])->format("m/d/Y")) < strtotime(date("m/d/Y"))){ 
                                    throw new \Exception("Certificado expirado.");
                                }
                            }
                        }
                    }    
                    
                } else {
                    throw new \Exception("Certificado do transmissor não encontrado. Favor carregar o certificado na tela 'Parametros do Reinf'.");
                }
            } else {
                throw new \Exception('Empresa não possui parâmetros para envio do Reinf cadastrados.');
            }
        } catch(\Exception $e) {
            throw $e;
        }
    }   
    
    public function enviarLote($eventosLote, $xmlLote) {
        try {
            
            $soapVar = new \SoapVar($xmlLote, XSD_ANYXML);
            
            $lotUri = $this->selecionaUrl();
            $this->saveXml($xmlLote, $this->_nrenvioTransmitir);
            $action = $this->_idOperacao != 'consulta' ? self::SOAPACTION_ENVIO_LOTE : self::SOAPACTION_CONSULTA;
            $resultado = $this->buildSoapCall(
                $this->_certKeyFile,
                $lotUri.'?singleWSDL',
                $action,
                $soapVar,
                $xmlLote
            );
            
            if ($resultado->ReceberLoteEventosResult) {
                $xmlResult = $resultado->ReceberLoteEventosResult->any;
                
                $this->saveXml($xmlResult, $this->_nrenvioTransmitir, false);
                $retorno = $this->_idOperacao != 'consulta' ?
                    $this->trataRetornoEnvioLote($eventosLote, $xmlResult)
                    : $this->tratarRetornoConsulta($eventosLote, $xmlResult);
                return $retorno;
            }  else {
                throw new \Exception($resultado->__toString());
            }
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function enviarLoteConsulta($eventosLote, $xmlLote, $reaberto = false) {
        try {
            
            $soapVar = new \SoapVar($xmlLote, XSD_ANYXML);
            
            $lotUri = $this->selecionaUrl();
            $this->saveXml($xmlLote, $this->_nrenvioTransmitir);
            //$action = $this->_idOperacao != 'consulta' ? self::SOAPACTION_ENVIO_LOTE : self::CONSULTA_LOTE_TEST;
        
            switch($eventosLote['NMEVENTO']){
                case self::R1000 :
                    $action = self::SOAPACTION_CONSULTA_R1000;
                    break;
                    
                case self::R2010 :
                    $action = self::SOAPACTION_CONSULTA_R2010;
                    break;
                    
                case self::R2020 :
                    $action = self::SOAPACTION_CONSULTA_R2020;
                    break;
                    
                case self::R2060 :
                    $action = self::SOAPACTION_CONSULTA_R2060;
                    break;
                    
                case self::R2098 : 
                    $action = self::SOAPACTION_CONSULTA_R2098;
                    break;
                    
                case self::R2099 : 
                    $action = self::SOAPACTION_CONSULTA_R2099;
                    break;
            }
            
            $resultado = $this->buildSoapCall(
                $this->_certKeyFile,
                $lotUri.'?singleWSDL',
                $action,
                $soapVar,
                $xmlLote
            );
            
            if ($resultado->ReceberLoteEventosResult) {
                $xmlResult = $resultado->ReceberLoteEventosResult->any;
                
                switch($eventosLote['NMEVENTO']){
                    
                    case self::R1000 :
                        $retorno = $this->tratarRetornoConsulta1000($eventosLote, $xmlResult);
                        break;
                    
                    case self::R2010 :
                        $retorno = $this->tratarRetornoConsulta2010($eventosLote, $xmlResult);
                        break;
                        
                    case self::R2020 :
                        $retorno = $this->tratarRetornoConsulta2020($eventosLote, $xmlResult);
                        break;
                        
                    case self::R2060 :
                        $retorno = $this->tratarRetornoConsulta2060($eventosLote, $xmlResult);
                        break;
                    
                    case self::R2098 : 
                        $retorno = $this->tratarRetornoConsulta2098($eventosLote, $xmlResult, $reaberto);
                        break;
                    
                    case self::R2099 : 
                        $retorno = $this->tratarRetornoConsulta2099($eventosLote, $xmlResult, $reaberto);
                        break;
                }
            
                return $retorno;
                
            }  else {
                throw new \Exception($resultado->__toString());
            }
        } catch(\Exception $e) {
            throw $e;
        }
    }
   
    public function enviarLimparBase($eventosLote, $xmlLote, $cdEmpresa, $idAmbiente) {
        try {
                
            $soapVar = new \SoapVar($xmlLote, XSD_ANYXML);
            
            $lotUri = $this->selecionaUrl();
            $this->saveXml($xmlLote, $this->_nrenvioTransmitir);
            $action = $this->_idOperacao != 'consulta' ? self::SOAPACTION_ENVIO_LOTE : self::SOAPACTION_CONSULTA;
            $resultado = $this->buildSoapCall(
                $this->_certKeyFile,
                $lotUri.'?singleWSDL',
                $action,
                $soapVar,
                $xmlLote
            );
            if ($resultado->ReceberLoteEventosResult) {
                $xmlResult = $resultado->ReceberLoteEventosResult->any;
            
                $codRetorno = null;
                $msg        = null;
                $xmlObj     = simplexml_load_string($xmlResult);
                $codRetorno = $xmlObj->retornoLoteEventos->status->cdStatus;
                
                if($codRetorno[0] == 0){
                    $cdRetorno = $xmlObj->retornoLoteEventos->retornoEventos->evento->Reinf->evtTotal->ideRecRetorno->ideStatus->cdRetorno;
                    if($cdRetorno[0] == 0){
                        $retorno = array(
                                    'COD' => $cdRetorno[0],
                                    'MSG' => 'Limpeza da base de dados para o contribuinte informado feita com sucesso.'
                                );
                        $this->factory->deletaEnvioEventoLimparBase($cdEmpresa, $idAmbiente);
                        return $retorno;
                    } else if($cdRetorno[0] == 1){
                        $regOcorrs = $xmlObj->retornoLoteEventos->retornoEventos->evento->Reinf->evtTotal->ideRecRetorno->ideStatus->regOcorrs;
                        foreach ($regOcorrs as $ocorrencia) {
                            $msg .= "\n* ". $ocorrencia->codResp . ' - ' . $ocorrencia->dscResp;
                            if ($ocorrencia->localErroAviso) {
                                $msg .= " | " . $ocorrencia->localErroAviso;
                            }
                        }
                        $retorno = array(
                                'COD' => $cdRetorno[0],
                                'MSG' => $msg
                        );
                        return $retorno;
                    }
                } else if($codRetorno[0] == 1){
                    $retorno = array(
                                'COD' => $codRetorno[0],
                                'MSG' => 'Erro'
                            );
                    return $retorno;
                }
            }  else {
                throw new \Exception($resultado->__toString());
            }
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function buildSoapCall($cert, $wsdl, $action, $soapVar, $xmlLote) {
        try {
            $msgSize = strlen($xmlLote);
            $parameters = [
                "Content-Type: text/xml;charset=UTF-8",
                "SOAPAction: $action",
                "Content-length: $msgSize"
            ];
            
            $oCurl = curl_init();
            curl_setopt($oCurl, CURLOPT_URL, $wsdl);
            curl_setopt($oCurl, CURLOPT_IPRESOLVE, CURL_IPRESOLVE_V4);
            curl_setopt($oCurl, CURLOPT_HEADER, 1);
            curl_setopt($oCurl, CURLOPT_SSL_VERIFYHOST, 0);
            curl_setopt($oCurl, CURLOPT_SSL_VERIFYPEER, 0);
            curl_setopt($oCurl, CURLOPT_SSLVERSION, 0);
            curl_setopt($oCurl, CURLOPT_SSLCERT, $cert);
            curl_setopt($oCurl, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($oCurl, CURLOPT_POST, true);
            curl_setopt($oCurl, CURLOPT_POSTFIELDS, $xmlLote);
            curl_setopt($oCurl, CURLOPT_HTTPHEADER, $parameters);
            $response = curl_exec($oCurl);
            $this->soaperror = curl_error($oCurl);
            $ainfo = curl_getinfo($oCurl);
            if (is_array($ainfo)) {
                $this->soapinfo = $ainfo;
            }
            $headsize = curl_getinfo($oCurl, CURLINFO_HEADER_SIZE);
            $httpcode = curl_getinfo($oCurl, CURLINFO_HTTP_CODE);
            curl_close($oCurl);
            $responseHead = trim(substr($response, 0, $headsize));
            $responseBody = trim(substr($response, $headsize));
            if ($httpcode == 404) {
                throw new \Exception('Erro 404 ao comunicar com a Sefaz.', 1);
            } else if ($httpcode == 500) {
                throw new \Exception($responseBody, 1);
            } else if ($httpcode != 200) {
                throw new \Exception('Erro inesperado. ERRO: '.$httpcode, 1);
            } else {
                $xml = new \SimpleXMLElement($responseBody);
                $domxml = new \DOMDocument();
                $domxml->preserveWhiteSpace = false;
                $domxml->formatOutput = true;
                $domxml->loadXML($xml->asXML());
                $result = $domxml->getElementsByTagName('Reinf')->item(0);
                $reinfXML = $result->ownerDocument->saveXML($result);
                
                $stdClass = new \StdClass;
                $reinf = new \StdClass;
                $reinf->any = $reinfXML;
                $stdClass->ReceberLoteEventosResult = $reinf;
                    
                return $stdClass;
            }
            
            
        } catch(\Exception $e) {
            throw $e;
        }
    }
   
    public function getChildNodeByName(\DOMNode $node, $name) {
        foreach ($node->childNodes as $childNode) {
            if ($childNode->nodeName == $name) {
                return $childNode;
            }
        }
        return null;
    }
   
    public function trataRetornoEnvioLote($eventosLote, $xmlResult) {
         try {
            /****************************************/
            // Atualmente situação 1-Erro | 0-Sucesso
            $codRetorno = null;
            $cdErro     = null;
            $msg        = null;
            $xmlObj     = simplexml_load_string($xmlResult);
            
            $codRetorno = $xmlObj->retornoLoteEventos->status->cdStatus;
            // Lote Enviado com Sucesso
            if($codRetorno != self::CD_ERRO){
                $msg     = MessagesFiscal::ENVIO_LOTE_SUCESSO;
                $eventos = $xmlObj->retornoLoteEventos->retornoEventos->evento;
                // Percorre o xml de retorno referente a cada evento e atualiza as informações de acordo com 
                // as informações retornadas
                for($i=0; $i<count($eventos); $i++ ){
                    $id = (String) $eventos[$i]->attributes();
                    $nrenvio          = (float)  substr($id,31);
                    
                    //Pega envio na base
                    $paramsEnvio = array(
                        "cdempresa" => $this->_cdempresa,
                        "nrenvio" => $nrenvio
                    );
                    $envio = $this->entityManager->getRepository(Envioeventoreinf::class)->findOneBy($paramsEnvio);
                    
                    if ($eventos[$i]->Reinf->evtTotal) {
                        
                        $nmEvento  = 'R-'.(String) $eventos[$i]->Reinf->evtTotal->infoRecEv->tpEv;
                                                            
                        $codRetornoEvento = (String) $eventos[$i]->Reinf->evtTotal->ideRecRetorno->ideStatus->cdRetorno;
                        if ($codRetornoEvento != self::CD_ERRO) {
                            $idStatus = $codRetornoEvento == self::CD_SUCESSO ? self::IDSTATUS_CONCLUIDO : self::IDSTATUS_PROCESSAMENTO;
                        } else if ($codRetornoEvento != self::CD_SUCESSO &&
                                    $this->_idOperacao == 'alteracao' &&
                                    $nmEvento == self::R1000) {
                            $idStatus = self::IDSTATUS_CONCLUIDO;
                        } else {
                            $idStatus = self::IDSTATUS_ERRO_ENVIO;
                        }
                                                            
                        $dtcompsemmovto   = self::R2099 == $nmEvento ? $this->ultimaCompetenciaSemMovimento($eventos[$i]) : null; 
                        
                        $dataEnvio = substr((String)$eventos[$i]->Reinf->evtTotal->infoRecEv->dhProcess, 0, 10);
                        $horaEnvio = substr((String)$eventos[$i]->Reinf->evtTotal->infoRecEv->dhProcess, 11, 8);
                        $dataHoraEnvio = $dataEnvio.' '.$horaEnvio;
                        $dataHoraEnvio = DateUtil::getDataDeString($dataHoraEnvio, 'Y-m-d H:i:s');
                        $dataHoraEnvio = $dataHoraEnvio->format('d/m/Y H:i:s');
                                                                        
                        if ($codRetornoEvento != self::CD_ERRO && $nmEvento == self::R2099) {
                            $recibo = (String)$eventos[$i]->Reinf->evtTotal->infoRecEv->nrProtEntr;
                        } else if ($codRetornoEvento != self::CD_ERRO) {
                            $recibo = (String)$eventos[$i]->Reinf->evtTotal->infoTotal->nrRecArqBase;
                        } else {
                            $recibo = $envio->getNrrecibo();
                        }             
                            
                        $dsobservacao = $codRetornoEvento != self::CD_ERRO ? $this->trataRetornoSemErro($eventos[$i]) 
                                        : $this->trataOcorrenciasEvento($eventos[$i]->Reinf->evtTotal->ideRecRetorno->ideStatus->regOcorrs);
                                        
                    }
                    
                    $this->factory->updateSituacaoEvento($idStatus, $dataHoraEnvio, $nrenvio, $dsobservacao, $id, $recibo, $dtcompsemmovto);
                    $paramsEvento = array(
                        "nrevento" => $envio->getNrevento()
                    );
                    $evento = $this->entityManager->getRepository(Eventoreinf::class)->findOneBy($paramsEvento);
                    
                    if (($this->_idOperacao == 'retificacao' || $this->_idOperacao == 'inclusao') && 
                        ($evento->getNmevento() == self::R2010 || $evento->getNmevento() == self::R2020) &&
                        $codRetornoEvento == self::CD_SUCESSO) {
                        $this->insereTabelaHistoricoEnvio($envio, $evento->getNmevento());
                    }
                    
                    if (($this->_idOperacao == 'exclusao' || ($this->_idOperacao == 'inclusao' && $evento->getNmevento() == self::R2098) ) && 
                            $codRetornoEvento == self::CD_SUCESSO) 
                    {
                        $observacaoExclusao = $evento->getNmevento() == self::R9000 ? 
                            MessagesFiscal::messagemEventoExcluido($envio->getNrenvioexclusao(), $this->operador) :
                            MessagesFiscal::messagemPeriodoReaberto($envio->getNrenvioexclusao(), $this->operador);
                        $newStatus = $this->_idOperacao == 'exclusao' ? 'X' : 'R';
                        $this->factory->updateEventoExcluido($envio->getNrenvioexclusao(), $newStatus, $observacaoExclusao);
                    }
                    
                    $this->entityManager->flush();
                }
            } else { // Erro no Envio do Lote
                $cdErro = (string) $xmlObj->retornoLoteEventos->status->cdStatus;
                $msg = $this->buildMsgErroLote($xmlResult);
            }
          
            return array(
                'COD' => $codRetorno,
                'MSG' => $msg,
            );
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function tratarRetornoConsulta($eventosLote, $xmlResult) {
        $naoUpdata = false;
        $retorno = array('COD' => '0');
        $xmlSalvo = null;
        
        $xmlObj = simplexml_load_string($xmlResult);
        $codStatus = (String) $xmlObj->evtTotalContrib->ideRecRetorno->ideStatus->cdRetorno;
        
        $msg = MessagesFiscal::CONSULTA_SUCESSO;
        
        $dataEnvio = substr((String) $xmlObj->evtTotalContrib->infoRecEv->dhProcess, 0, 10);
        $horaEnvio = substr((String) $xmlObj->evtTotalContrib->infoRecEv->dhProcess, 11, 8);
        $dataHoraEnvio = $dataEnvio.' '.$horaEnvio;
        $dataHoraEnvio = DateUtil::getDataDeString($dataHoraEnvio, 'Y-m-d H:i:s');
        $dataHoraEnvio = $dataHoraEnvio->format('d/m/Y H:i:s');
        
        if ($codStatus != self::CD_ERRO) {
            $descricao = 'Consulta realizada com sucesso.';
            $idstatus = 'C';
            
            $paramsEnvio = array(
                "cdempresa" => $this->_cdempresa,
                "nrenvio" => $this->_nrenvioTransmitir
            );
            $envio = $this->entityManager->getRepository(Envioeventoreinf::class)->findOneBy($paramsEnvio);
            
            $paramsEvento = array(
                "nrevento" => $envio->getNrevento()
            );
            $evento = $this->entityManager->getRepository(Eventoreinf::class)->findOneBy($paramsEvento);
            
            if ($evento->getNmevento() == self::R2099) {
                if($codStatus != 2) {
                    $xmlSalvo = $this->saveXmlInFileUplouder('/tmp/recebidos/'.$envio->getNrenvio().'.xml');
                    
                    if ($xmlSalvo != false) {
                        $msg = $msg.'<br> Foi gerado um relátorio com o resultado da consulta.';
                        
                        $paramsEmpresa = array(
                            "cdempresa" => $envio->getCdempresa()
                        );
                        $empresa = $this->entityManager->getRepository(Empresa::class)->findOneBy($paramsEmpresa);
                        
                        $periodo = $envio->getDtmescompetenc()->format('m/Y');
                        $empresaToReport = $empresa->getCdempresa().' - '.$empresa->getNmempresa();
                        
                        
                        $localXml = "$this->_fileServerUrl/$xmlSalvo";
                        
                        $reportUrl = $this->geraRelatorioFechamento($empresaToReport, $periodo, $localXml);
                        $retorno['REPORT_URL'] = $reportUrl;   
                    } else {
                        $msg = $msg.'<br> Porém o arquio xml de retorno não foi salvo na pasta temporaria. É necessário que ele seja salvo para geração do relatório.';
                        $msg = $msg.'<br> Consulte novamente, se o problema persistir, contate o administrador do sistema.';
                        $naoUpdata = true;
                    }
                }
                else {
                    $descricao = "Evento em processamento. Tente novamente mais tarde.";
                    $idstatus  = "M";
                }
            }
            
        } else {
            $erroDescricao = (String) $xmlObj->evtTotalContrib->ideRecRetorno->ideStatus->regOcorrs->dscResp;
            
            $descricao = 'Erro na consulta. '.$erroDescricao;
            
            $idstatus = $eventosLote['IDSTATUS'];
        }
        
        if (!$naoUpdata) {
            $this->factory->updateSituacaoEvento(
                $idstatus,
                $eventosLote['DTENVIO'],
                $eventosLote['NRENVIO'],
                $descricao,
                $eventosLote['IDEVENTO'],
                $eventosLote['NRRECIBO'],
                $eventosLote['DTCOMPSEMMOVTO'],
                $dataHoraEnvio,
                $xmlSalvo);
        }
                
        $this->entityManager->flush();
        
        $retorno['MSG'] = $msg;
        return $retorno;
    }

    public function tratarRetornoConsulta1000($eventosLote, $xmlResult) {
        $retorno = array('COD' => '0');
        
        $xmlObj = simplexml_load_string($xmlResult);
        $codStatus = (String) $xmlObj->ideStatus->cdRetorno;
        
        $msg = MessagesFiscal::CONSULTA_SUCESSO;
        
        if ($codStatus != self::CD_ERRO) {
            $retorno['COD'] = $codStatus;
            $descricao = 'Consulta realizada com sucesso.';
            
            if($codStatus != 2) {
                $msg = $msg.'<br>'. (String) $xmlObj->ideStatus->descRetorno.'.';
            }
            
        } else if ($codStatus == self::CD_ERRO) {
            $retorno['COD'] = $codStatus;
            $descricao = 'Consulta realizada com sucesso.';
            
            if($codStatus != 2) {
                $msg = $msg.'<br>'. (String) $xmlObj->ideStatus->descRetorno.'.'.
                '<br>'.' Início do Período '.substr((String) $xmlObj->retornoEventos->evento->iniValid, 5, 2).'/'.substr((String) $xmlObj->retornoEventos->evento->iniValid, 0, 4).' .';
                if((String) $xmlObj->retornoEventos->evento->fimValid != null){
                    $msg = $msg.'<br>'.' Início do Período '.substr((String) $xmlObj->retornoEventos->evento->fimValid, 5, 2).'/'.substr((String) $xmlObj->retornoEventos->evento->fimValid, 0, 4).' .';
                }
            }
            
        } else {
            $erroDescricao = (String) $xmlObj->ideStatus->descRetorno;
            
            $descricao = 'Erro na consulta.'.'<br>'.$erroDescricao;
            
        }
        
        $this->entityManager->flush();
        
        $retorno['MSG'] = $msg;
        return $retorno;
    }
    
    public function tratarRetornoConsulta2010($eventosLote, $xmlResult) {
        $retorno = array('COD' => '0');
        
        $xmlObj = simplexml_load_string($xmlResult);
        $codStatus = (String) $xmlObj->ideStatus->cdRetorno;
        
        $msg = MessagesFiscal::CONSULTA_SUCESSO;
        
        if ($codStatus != self::CD_ERRO) {
            $retorno['COD'] = $codStatus;
            $descricao = 'Consulta realizada com sucesso.';
            
            if($codStatus != 2) {
                $msg = $msg.'<br>'. (String) $xmlObj->ideStatus->descRetorno.'.';
            }
            
        } else if ($codStatus == self::CD_ERRO) {
            $retorno['COD'] = $codStatus;
            $descricao = 'Consulta realizada com sucesso.';
            
            if($codStatus != 2) {
                $msg = $msg.'<br>'. (String) $xmlObj->ideStatus->descRetorno.'.';
                
                $i = 0;
                $lastDate = array();
                
                foreach($xmlObj->retornoEventos->evento as $eventos){
                    array_push($lastDate, (String) $eventos->dtHoraRecebimento);
                }
                
                foreach($xmlObj->retornoEventos->evento as $eventos){
                    if($eventos->dtHoraRecebimento == max($lastDate)){
                        $id = (String) $eventos->attributes()->id[0];
                        $dataEnvio = substr((String) $eventos->dtHoraRecebimento, 6, 2) .'/'. substr((String) $eventos->dtHoraRecebimento, 4, 2) .'/'. substr((String) $eventos->dtHoraRecebimento, 0, 4);
                        $horaEnvio = substr((String) $eventos->dtHoraRecebimento, 8, 2) .':'. substr((String) $eventos->dtHoraRecebimento, 10, 2) .':'. substr((String) $eventos->dtHoraRecebimento, 12, 2);
                        $dataHoraEnvio = $dataEnvio.' '.$horaEnvio;
                        //$dataHoraEnvio = DateUtil::getDataDeString($dataHoraEnvio, 'Y-m-d H:i:s');
                        //$dataHoraEnvio = $dataHoraEnvio->format('d/m/Y H:i:s');
                        $nrRecibo = (String) $eventos->nrRecibo;
                        $situacaoEvento = (String) $eventos->situacaoEvento;
                        $aplicacaoRecepcao = (String) $eventos->aplicacaoRecepcao;
                        if ($situacaoEvento === '3'){
                            $idStatus = 'X';
                            $msg = 'O envio encontrado foi Excluido.'.'<br>'.'Favor enviar novamente um novo evento.';
                        } else if ($situacaoEvento == '1' || $situacaoEvento == '2'){
                            $idStatus = 'C';
                            $this->factory->updateSituacaoEvento($idStatus, $dataHoraEnvio, $eventosLote['NRENVIO'], $eventosLote['DSOBSERVACAO'], $id, $nrRecibo, $eventosLote['DTCOMPSEMMOVTO']);
                        }
                        
                        $this->factory->updateSituacaoEvento($idStatus, $dataHoraEnvio, $eventosLote['NRENVIO'], $eventosLote['DSOBSERVACAO'], $id, $nrRecibo, $eventosLote['DTCOMPSEMMOVTO']);
                    }
                }
                
            }
            
        } else {
            $erroDescricao = (String) $xmlObj->ideStatus->descRetorno;
            
            $descricao = 'Erro na consulta.'.'<br>'.$erroDescricao;
            
        }
        
        $this->entityManager->flush();
        
        $retorno['MSG'] = $msg;
        return $retorno;
    }
    
    public function tratarRetornoConsulta2020($eventosLote, $xmlResult) {
        $retorno = array('COD' => '0');
        
        $xmlObj = simplexml_load_string($xmlResult);
        $codStatus = (String) $xmlObj->ideStatus->cdRetorno;
        
        $msg = MessagesFiscal::CONSULTA_SUCESSO;
        
        if ($codStatus != self::CD_ERRO) {
            $retorno['COD'] = $codStatus;
            $descricao = 'Consulta realizada com sucesso.';
            
            if($codStatus != 2) {
                $msg = $msg.'<br>'. (String) $xmlObj->ideStatus->descRetorno.'.';
            }
            
        } else if ($codStatus == self::CD_ERRO) {
            $retorno['COD'] = $codStatus;
            $descricao = 'Consulta realizada com sucesso.';
            
            if($codStatus != 2) {
                $msg = $msg.'<br>'. (String) $xmlObj->ideStatus->descRetorno.'.';
                
                $i = 0;
                $lastDate = array();
                
                foreach($xmlObj->retornoEventos->evento as $eventos){
                    array_push($lastDate, (String) $eventos->dtHoraRecebimento);
                }
                
                foreach($xmlObj->retornoEventos->evento as $eventos){
                    if($eventos->dtHoraRecebimento == max($lastDate)){
                        $id = (String) $eventos->attributes()->id[0];
                        $dataEnvio = substr((String) $eventos->dtHoraRecebimento, 6, 2) .'/'. substr((String) $eventos->dtHoraRecebimento, 4, 2) .'/'. substr((String) $eventos->dtHoraRecebimento, 0, 4);
                        $horaEnvio = substr((String) $eventos->dtHoraRecebimento, 8, 2) .':'. substr((String) $eventos->dtHoraRecebimento, 10, 2) .':'. substr((String) $eventos->dtHoraRecebimento, 12, 2);
                        $dataHoraEnvio = $dataEnvio.' '.$horaEnvio;
                        //$dataHoraEnvio = DateUtil::getDataDeString($dataHoraEnvio, 'Y-m-d H:i:s');
                        //$dataHoraEnvio = $dataHoraEnvio->format('d/m/Y H:i:s');
                        $nrRecibo = (String) $eventos->nrRecibo;
                        $situacaoEvento = (String) $eventos->situacaoEvento;
                        $aplicacaoRecepcao = (String) $eventos->aplicacaoRecepcao;
                        if ($situacaoEvento === '3'){
                            $idStatus = 'X';
                            $msg = 'O envio encontrado foi Excluido.'.'<br>'.'Favor enviar novamente um novo evento.';
                        } else if ($situacaoEvento == '1' || $situacaoEvento == '2'){
                            $idStatus = 'C';
                            $this->factory->updateSituacaoEvento($idStatus, $dataHoraEnvio, $eventosLote['NRENVIO'], $eventosLote['DSOBSERVACAO'], $id, $nrRecibo, $eventosLote['DTCOMPSEMMOVTO']);
                        }
                        
                        
                        $this->factory->updateSituacaoEvento($idStatus, $dataHoraEnvio, $eventosLote['NRENVIO'], $eventosLote['DSOBSERVACAO'], $id, $nrRecibo, $eventosLote['DTCOMPSEMMOVTO']);
                    }
                }
                
            }
            
        } else {
            $erroDescricao = (String) $xmlObj->ideStatus->descRetorno;
            
            $descricao = 'Erro na consulta.'.'<br>'.$erroDescricao;
            
        }
        
        $this->entityManager->flush();
        
        $retorno['MSG'] = $msg;
        return $retorno;
    }
    
    public function tratarRetornoConsulta2060($eventosLote, $xmlResult) {
        $retorno = array('COD' => '0');
        
        $xmlObj = simplexml_load_string($xmlResult);
        $codStatus = (String) $xmlObj->ideStatus->cdRetorno;
        
        $msg = MessagesFiscal::CONSULTA_SUCESSO;
        
        if ($codStatus != self::CD_ERRO) {
            $retorno['COD'] = $codStatus;
            $descricao = 'Consulta realizada com sucesso.';
            
            if($codStatus != 2) {
                $msg = $msg.'<br>'. (String) $xmlObj->ideStatus->descRetorno.'.';
            }
            
        } else if ($codStatus == self::CD_ERRO) {
            $retorno['COD'] = $codStatus;
            $descricao = 'Consulta realizada com sucesso.';
            
            if($codStatus != 2) {
                $msg = $msg.'<br>'. (String) $xmlObj->ideStatus->descRetorno.'.';
                
                $i = 0;
                $lastDate = array();
                
                foreach($xmlObj->retornoEventos->evento as $eventos){
                    array_push($lastDate, (String) $eventos->dtHoraRecebimento);
                }
                
                foreach($xmlObj->retornoEventos->evento as $eventos){
                    if($eventos->dtHoraRecebimento == max($lastDate)){
                        $id = (String) $eventos->attributes()->id[0];
                        $dataEnvio = substr((String) $eventos->dtHoraRecebimento, 6, 2) .'/'. substr((String) $eventos->dtHoraRecebimento, 4, 2) .'/'. substr((String) $eventos->dtHoraRecebimento, 0, 4);
                        $horaEnvio = substr((String) $eventos->dtHoraRecebimento, 8, 2) .':'. substr((String) $eventos->dtHoraRecebimento, 10, 2) .':'. substr((String) $eventos->dtHoraRecebimento, 12, 2);
                        $dataHoraEnvio = $dataEnvio.' '.$horaEnvio;
                        //$dataHoraEnvio = DateUtil::getDataDeString($dataHoraEnvio, 'Y-m-d H:i:s');
                        //$dataHoraEnvio = $dataHoraEnvio->format('d/m/Y H:i:s');
                        $nrRecibo = (String) $eventos->nrRecibo;
                        $situacaoEvento = (String) $eventos->situacaoEvento;
                        $aplicacaoRecepcao = (String) $eventos->aplicacaoRecepcao;
                        if ($situacaoEvento === '3'){
                            $idStatus = 'X';
                            $msg = 'O envio encontrado foi Excluido.'.'<br>'.'Favor enviar novamente um novo evento.';
                        } else if ($situacaoEvento == '1' || $situacaoEvento == '2'){
                            $idStatus = 'C';
                            $this->factory->updateSituacaoEvento($idStatus, $dataHoraEnvio, $eventosLote['NRENVIO'], $eventosLote['DSOBSERVACAO'], $id, $nrRecibo, $eventosLote['DTCOMPSEMMOVTO']);
                        }
                    }
                }
                
            }
            
        } else {
            $erroDescricao = (String) $xmlObj->ideStatus->descRetorno;
            
            $descricao = 'Erro na consulta.'.'<br>'.$erroDescricao;
            
        }
        
        $this->entityManager->flush();
        
        $retorno['MSG'] = $msg;
        return $retorno;
    }
    
    public function tratarRetornoConsulta2098($eventosLote, $xmlResult, $reaberto) {
        $retorno = array('COD' => '0');
        
        $xmlObj = simplexml_load_string($xmlResult);
        
        $codStatus = (String) $xmlObj->ideStatus->cdRetorno;
        
        $msg = MessagesFiscal::CONSULTA_SUCESSO;
        
        if ($codStatus != self::CD_ERRO) {
            $retorno['COD'] = $codStatus;
            $descricao = 'Consulta realizada com sucesso.';
            
            if($codStatus != 2) {
                $msg = $msg.'<br>'. (String) $xmlObj->ideStatus->descRetorno.'.';
            }
            
        } else if ($codStatus == self::CD_ERRO) {
            $retorno['COD'] = $codStatus;
            $descricao = 'Consulta realizada com sucesso.';
            
            if($codStatus != 2) {
                $msg = $msg.'<br>'. (String) $xmlObj->ideStatus->descRetorno.'.';
                
                $i = 0;
                $lastDate = array();
                
                foreach($xmlObj->retornoEventos->evento as $eventos){
                    array_push($lastDate, (String) $eventos->dtHoraRecebimento);
                }
                
                foreach($xmlObj->retornoEventos->evento as $eventos){
                    if($eventos->dtHoraRecebimento == max($lastDate)){
                        $id = (String) $eventos->attributes()->id[0];
                        $dataEnvio = substr((String) $eventos->dtHoraRecebimento, 6, 2) .'/'. substr((String) $eventos->dtHoraRecebimento, 4, 2) .'/'. substr((String) $eventos->dtHoraRecebimento, 0, 4);
                        $horaEnvio = substr((String) $eventos->dtHoraRecebimento, 8, 2) .':'. substr((String) $eventos->dtHoraRecebimento, 10, 2) .':'. substr((String) $eventos->dtHoraRecebimento, 12, 2);
                        $dataHoraEnvio = $dataEnvio.' '.$horaEnvio;
                        //$dataHoraEnvio = DateUtil::getDataDeString($dataHoraEnvio, 'Y-m-d H:i:s');
                        //$dataHoraEnvio = $dataHoraEnvio->format('d/m/Y H:i:s');
                        $nrRecibo = (String) $eventos->nrRecibo;
                        $situacaoEvento = (String) $eventos->situacaoEvento;
                        $aplicacaoRecepcao = (String) $eventos->aplicacaoRecepcao;
                        if($reaberto == false){
                            if ($situacaoEvento === '3'){
                                $idStatus = 'X';
                                $msg = 'O envio encontrado foi Excluido.'.'<br>'.'Favor enviar novamente um novo evento.';
                            } else if ($situacaoEvento == '1' || $situacaoEvento == '2'){
                                $idStatus = 'C';
                                $this->factory->updateSituacaoEvento($idStatus, $dataHoraEnvio, $eventosLote['NRENVIO'], $eventosLote['DSOBSERVACAO'], $id, $nrRecibo, $eventosLote['DTCOMPSEMMOVTO']);
                            }
                        } else if($reaberto == true){
                            $eventoR2098 = $this->factory->verificaR2098Reaberto($eventosLote['CDEMPRESA'], substr($eventosLote['DTMESCOMPETENC'], 0, 10));
                            if(!empty($eventoR2098)){
                                $idStatus = 'C';
                                $this->factory->updateSituacaoEvento($idStatus, $dataHoraEnvio, $eventoR2098['NRENVIO'], isset($eventoR2098['DSOBSERVACAO']) ? $eventoR2098['DSOBSERVACAO'] : null, $id, $nrRecibo, $eventosLote['DTCOMPSEMMOVTO']);
                            }
                        }
                    }
                }
                
            }
            
        } else {
            
            $erroDescricao = (String) $xmlObj->ideStatus->descRetorno;
            
            $descricao = 'Erro na consulta.'.'<br>'.$erroDescricao;
            
        }
        
        $this->entityManager->flush();
        
        $retorno['MSG'] = $msg;
        return $retorno;
    }
    
    public function tratarRetornoConsulta2099($eventosLote, $xmlResult, $reaberto) {
        $retorno = array('COD' => '0');
        
        $xmlObj = simplexml_load_string($xmlResult);
        $codStatus = (String) $xmlObj->ideStatus->cdRetorno;
        
        $msg = MessagesFiscal::CONSULTA_SUCESSO;
        
        if ($codStatus != self::CD_ERRO) {
            $retorno['COD'] = $codStatus;
            $descricao = 'Consulta realizada com sucesso.';
            
            if($codStatus != 2) {
                $msg = $msg.'<br>'. (String) $xmlObj->ideStatus->descRetorno.'.';
            }
            
        } else if ($codStatus == self::CD_ERRO) {
            
            $retorno['COD'] = $codStatus;
            $descricao = 'Consulta realizada com sucesso.';
            
            if($codStatus != 2) {
                $i = 0;
                $lastDate = array();
                foreach($xmlObj->retornoEventos->evento as $eventos){
                    array_push($lastDate, (String) $eventos->dtHoraRecebimento);
                }
                
                foreach($xmlObj->retornoEventos->evento as $eventos){
                    if($eventos->dtHoraRecebimento == max($lastDate)){
                        $id = (String) $eventos->attributes()->id[0];
                        $dataEnvio = substr((String) $eventos->dtHoraRecebimento, 6, 2) .'/'. substr((String) $eventos->dtHoraRecebimento, 4, 2) .'/'. substr((String) $eventos->dtHoraRecebimento, 0, 4);
                        $horaEnvio = substr((String) $eventos->dtHoraRecebimento, 8, 2) .':'. substr((String) $eventos->dtHoraRecebimento, 10, 2) .':'. substr((String) $eventos->dtHoraRecebimento, 12, 2);
                        $dataHoraEnvio = $dataEnvio.' '.$horaEnvio;
                        $nrProtocolo = (String) $eventos->nrProtocolo;
                        $nrRecibo = (String) $eventos->nrRecibo;
                        $verificaR2099Reaberto = $this->factory->verificaR2099Reaberto($id, $nrProtocolo, 'R');
                        
                        if(empty($verificaR2099Reaberto) && $reaberto == false){
                            $situacaoEvento = (String) $eventos->situacaoEvento;
                            $aplicacaoRecepcao = (String) $eventos->aplicacaoRecepcao;
                            if ($situacaoEvento === '3'){
                                $idStatus = 'X';
                                $msg = 'O envio encontrado foi Excluido.'.'<br>'.'Favor enviar novamente um novo evento.';
                            } else if ($situacaoEvento == '1' || $situacaoEvento == '2'){
                                $idStatus = 'C';
                                $msg = $msg.'<br>'. (String) $xmlObj->ideStatus->descRetorno.'.';
                                $this->factory->updateSituacaoEvento($idStatus, $dataHoraEnvio, $eventosLote['NRENVIO'], $eventosLote['DSOBSERVACAO'], $id, $nrProtocolo, $eventosLote['DTCOMPSEMMOVTO']);
                            }
                        } else if ($reaberto == true){
                            if(count($xmlObj->retornoEventos->evento) === 1){
                                $verificaR2099Reaberto = $this->factory->verificaR2099Reaberto($id, $nrProtocolo, 'R');
                                if(empty($verificaR2099Reaberto)){
                                    $idStatus = 'R';
                                    $msg = $msg.'<br>'. (String) $xmlObj->ideStatus->descRetorno.'.';
                                    $msg = $msg.'<br>'.'Existe um evento R2098 concluído registado no E-CAC. Faça a transimissão de um novo evento 2099.';
                                    $this->factory->updateSituacaoEvento($idStatus, $dataHoraEnvio, $eventosLote['NRENVIO'], $eventosLote['DSOBSERVACAO'], $id, $nrProtocolo, $eventosLote['DTCOMPSEMMOVTO']);
                                } else {
                                    $msg = $msg.'<br>'. (String) $xmlObj->ideStatus->descRetorno.'.';
                                    $msg = $msg.'<br>'.'Existe um evento R2098 concluído registado no E-CAC. Faça a transimissão de um novo evento 2099.';
                                }
                            } else if(count($xmlObj->retornoEventos->evento) > 1){
                                $idStatus = 'C';
                                $msg = $msg.'<br>'. (String) $xmlObj->ideStatus->descRetorno.'.';
                                $this->factory->updateSituacaoEvento($idStatus, $dataHoraEnvio, $eventosLote['NRENVIO'], $eventosLote['DSOBSERVACAO'], $id, $nrProtocolo, $eventosLote['DTCOMPSEMMOVTO']);
                            }
                        } else {
                            if(($eventosLote['IDSTATUS'] == 'P') && (count($xmlObj->retornoEventos->evento) === 1)){
                                $msg = $msg.'<br>'.'Existe um evento 2099 reaberto, e o atual 2099 está pendente. Favor realizar o envio do evento pendente antes de realizar a consulta.';
                            } else {
                                $situacaoEvento = (String) $eventos->situacaoEvento;
                                $aplicacaoRecepcao = (String) $eventos->aplicacaoRecepcao;
                                if ($situacaoEvento === '3'){
                                    $idStatus = 'X';
                                    $msg = 'O envio encontrado foi Excluido.'.'<br>'.'Favor enviar novamente um novo evento.';
                                } else if ($situacaoEvento == '1' || $situacaoEvento == '2'){
                                    $idStatus = 'C';
                                    $msg = $msg.'<br>'. (String) $xmlObj->ideStatus->descRetorno.'.';
                                    $this->factory->updateSituacaoEvento($idStatus, $dataHoraEnvio, $eventosLote['NRENVIO'], $eventosLote['DSOBSERVACAO'], $id, $nrProtocolo, $eventosLote['DTCOMPSEMMOVTO']);
                                }   
                            }
                        }
                    }
                }
                
            }
            
        } else {
            $erroDescricao = (String) $xmlObj->ideStatus->descRetorno;
            
            $descricao = 'Erro na consulta.'.'<br>'.$erroDescricao;
            
        }
        
        $this->entityManager->flush();
        
        $retorno['MSG'] = $msg;
        return $retorno;
    }
    
    public function buildMsgErroLote($xmlResponse) {
        try {
            $xmlObj = simplexml_load_string($xmlResponse);
            $msg = '';
            $msg .=  $xmlObj->retornoLoteEventos->status->descRetorno;
            foreach ($xmlObj->retornoLoteEventos->status->dadosRegistroOcorrenciaLote->ocorrencias as $ocorrencia) {
                $msg .= "\n[" . $this->_tipoOcorRetorno[(string)$ocorrencia->tipo] . '] ' . $ocorrencia->codigo . '-' . $ocorrencia->descricao;
                if ($ocorrencia->localizacao) {
                    $msg .= " (" . $ocorrencia->localizacao . ")";
                }
            }

            return $msg;
        } catch(\Exception $e) {
            throw $e;
        }
    }
   
    // Trata todas as ocorrencias de um evento
    public function trataOcorrenciasEvento($ocorrencias){
        $msg = '';
        for($i=0; $i<count($ocorrencias); $i++ ){
            // Verifica se a tag localização do erro está presente e não esta vazia no xml de retorno
            $existeLocalizacao = ( isset($ocorrencias[$i]->localErroAviso) && (!empty ($ocorrencias[$i]->localErroAviso)) );
            
            $localizacao       = $existeLocalizacao ? ' | '.'Localização: '. explode (' - ' ,(String)$ocorrencias[$i]->localErroAviso[0])[1] : '';
            $msg               = $msg. '*'.$ocorrencias[$i]->codResp.' - '. $ocorrencias[$i]->dscResp.$localizacao."\n";
        }
        // tamanho maximo do campo de observação.
        return substr($msg, 0,4000);
        
    }

    public function insereTabelaHistoricoEnvio($envio, $nmevento) {
        $sequencial = $this->newCode->getCodeByTableName('NRSEQHISTENVIO', $this->nrorg, 1, 10);
        
        $envio2 = $this->factory->getEnvio($envio->getNrenvio());
        
        $paramsNota = array(
            'CDPARTICIPANTE' => $envio->getCdestabreinf(),
            'NRSEQUMAPAPU'   => $envio2['NRSEQUMAPAPU']
            // Descmentar ese bloco de apagar o bloco acima apos o campo de Nrsequmapapu for homologado na entidade.
            // 'NRSEQUMAPAPU'   => $envio->getNrsequmapapu()
        );
        $notas = $this->factory->notas2010Ou2020($paramsNota);
        
        foreach ($notas as $nota) {
            $params = array(
                'NRSEQHISTENVIO' => $sequencial,
                'NRENVIO' => $envio->getNrenvio(),
                'NRLANCTONF' => $nota['NRLANCTONF'],
                'CDFILIAL' => $nota['CDFILIAL'],
                'IDOPERACAO' => $this->_idOperacao == 'inclusao' ? 'I' : 'R',
                'NRORG' => $this->nrorg,
                'NRORGINCLUSAO' => $this->nrorg,
                'CDOPERINCLUSAO' => $this->operador
            );
            
            $this->factory->insereHistenvio($params);
        }
            
    }    

   /***************************************************
   *                Funções Auxilares                 *
   *                                                  *
   ***************************************************/
    private function trataRetornoSemErro($evento) {
        $rentorno = (String) $evento->Reinf->evtTotal->ideRecRetorno->ideStatus->descRetorno;
        if ($rentorno == 'EM PROCESSAMENTO') {
            return 'Evento em processamento.';
        } else if ($rentorno == 'SUCESSO') {
            return 'Evento transmitido com sucesso.';
        } else {
            return $rentorno;
        }
    }
   
    private function selecionaUrl() {
        if ($this->_tpAmb == 1) {
            return $this->_idOperacao != 'consulta' ? self::ENVIAR_LOTE_PROD : self::CONSULTA_LOTE_PROD;
        } else {
            return $this->_idOperacao != 'consulta' ? self::ENVIAR_LOTE_TEST : self::CONSULTA_LOTE_TEST;
        }
    }
   
    private function saveXml($xml, $nomeXml, $envioPath = true) {
        try {
            $complemento = $envioPath ? 'enviados' : 'recebidos';
            $path = '/tmp/'.$complemento;
            if (!is_dir($path)) {
                mkdir($path, 0775, true);
                chmod($path, 0775);
            }
            file_put_contents($path.'/'.$nomeXml.'.xml', $xml);
        } catch (\Exception $e) {}
    }
   
    // Aplica máscara de inscrição (CNPJ ou CPF) nos registros buscados.
    public function aplicarMascaraInscricao($rows,$nomeCampoInscricao) {
        foreach ($rows as &$row) {
            if (strlen($row[$nomeCampoInscricao]) == 14) {
                $row[$nomeCampoInscricao] = $this->masks->mask($row[$nomeCampoInscricao], '##.###.###/####-##');
            }
            else if (strlen($row[$nomeCampoInscricao])==11) {
                $row[$nomeCampoInscricao] = $this->masks->mask($row[$nomeCampoInscricao], '###.###.###-##');
            }
        }
        return $rows;
    }

    public function getParamReinf($cdempresa){
        return $this->factory->getParamReinf($cdempresa);
    }
    
    public function detalhamentoR2060($row){
        try{
            $dtinicial  = DateUtil::getPrimeiroDiaDoMes( $this->formataData($row['DTMESCOMPETENC']));
            $dtfinal    = DateUtil::getUltimoDiaDoMes( $this->formataData($row['DTMESCOMPETENC']));
            $dtinicial = $dtinicial->format("d/m/Y");
            $dtfinal = $dtfinal->format("d/m/Y");
            
            $vrexportacao = $this->factory->getVrexportacao($dtinicial, $dtfinal, $row['CDEMPRESA']);
            $detalhamentoR2060 = $this->factory->getDetalhamentoR2060($vrexportacao[0]['V_VREXPORTACAO'], $dtinicial, $dtfinal, $row['CDEMPRESA']);
            
            return $detalhamentoR2060;
            
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function verificaEventoServicosPeriodo($cdempresa, $dtmescompetenc, $nrevento){
        return $this->factory->verificaEventoServicosPeriodo($cdempresa, $dtmescompetenc, $nrevento);
    }
    
    private function getOperacaoEvento($idOperacao) {
        if ($idOperacao == 'I') {
            return 'inclusao';
        }
        if ($idOperacao == 'A') {
            return 'alteracao';
        }
        if ($idOperacao == 'E') {
            return 'exclusao';
        }
        if ($idOperacao == 'C') {
            return 'consulta';
        }
        if ($idOperacao == 'R') {
            return 'retificacao';
        }
        return null;
    }
 
    private function formataData($data) {
        if (!$data) {
            return null;
        }

        // Verifica se está com o formato de data-hora.
        if (strlen($data) == 19) {
            $data = DateUtil::truncateString($data);
        }
        $data = DateUtil::getDataDeString($data);
        $data = DateUtil::truncateData($data);

        return $data;
    }
    
    /*
        Função Exlusiva para o evento de Fechamento (R-2099).
        Informar a primeira competência a partir da qual não houve movimento, 
        cuja situação perdura até a competência atual.
    */
    private function ultimaCompetenciaSemMovimento ($evento){
        $id      = (String) $evento->attributes();
        $nrenvio = (float)  substr($id,31);
        
        $envio        = $this->factory->getEnvio($nrenvio);
        $movimentacao = $this->factory->verificaMovimentecaoPeriodo($envio['CDEMPRESA'], $envio['DSVERSAO'], $envio['DTMESCOMPETENC'])['MOVIMENTACAO'];
        
        if($movimentacao ==='N'){
            $maiorData = $this->factory->verificaMaiorDataSemMovimentacao($envio['CDEMPRESA'], $envio['NREVENTO'], $envio['DTMESCOMPETENC'])['DTCOMPSEMMOVTO'];
            if($maiorData)
                return $maiorData;
            else 
                return $envio['DTMESCOMPETENC'];
        }else if ($movimentacao ==='S')
            return null;
            
        
    }
    
    public function nrenvioR1000vigente($cdempresa, $ambiente, $dtInicial, $dtFinal) {
        $paramsEnvio = array(
            "CDEMPRESA" => $cdempresa,
			"IDAMBIENTE" => $ambiente,
			"DTINICIAL" => $dtInicial, 
			"DTFINAL" => $dtFinal
        );
        $EnvioR1000 = $this->factory->R1000Vigente($paramsEnvio);
        
        if (empty($EnvioR1000)) {
            throw new \Exception(MessagesFiscal::R1000_INEXISTENTE);
        } else {
            return $EnvioR1000['NRENVIO'];
        }
    }
    
    public function podeInserirR100($cdempresa, $ambiente, $dtInicial, $dtFinal) {
        $params = array(
            "CDEMPRESA" => $cdempresa,
            "IDAMBIENTE" => $ambiente,
            "DTINICIAL" => $dtInicial,
            "DTFINAL" => $dtFinal
        );
        $EnvioR1000 = $this->factory->R1000Vigente($params);
        $paramEmpresa = $this->entityManager->getRepository(Paramempreinf::class)->findOneBy($params);
        
        if( empty($EnvioR1000) ) {
            return true;
        } else if( !empty($EnvioR1000) || !$EnvioR1000['DTFINVALIDADE'] ) {
            throw new \Exception(MessagesFiscal::R1000_SEM_VALIDADE, 1);
        } else if( !empty($EnvioR1000) && $EnvioR1000['DTFINVALIDADE'] ) {
            $dataFinalR1000 = DateUtil::getDataDeString($EnvioR1000['DTFINVALIDADE']);
            if (DateUtil::comparaDatas($dataFinalR1000, $paramEmpresa->getDtfinvalidade, 3)) {
                throw new \Exception(MessagesFiscal::R1000_EXISTENTE_PARA_VALIDADE, 1);
            }
        } else {
            return true;
        }
        
    }
    
    private function geraRelatorioFechamento($empresa, $periodo, $localXml) {
        return $this->reportService->createRemoteReport(
            'BIRT', 
            self::R2099REPORT, 
            array(
                'EMPRESA' => $empresa,
                'PERIODO' => $periodo,
                'URL_XML' => $localXml
            )
        );
    }
    
    private function saveXmlInFileUplouder($tmpFile) {
        
        if (file_exists($tmpFile)) {
            $file = file_get_contents($tmpFile);
            $file = base64_encode($file);
        
            $file = 'data: '.mime_content_type($tmpFile).';base64,'.$file;
            
            $lastModify = filectime($tmpFile);
        
            $file = array(
                "name" => time().'.xml', 
                "lastModified" => $lastModify,
                "lastModifiedDate" => '',
                "webkitRelativePath" => '',
                "type" => '',
                "size"  => filesize($tmpFile),
                "b64File" => $file);
                
            $row['FILE']    = array();
            $row['FILE'][0] = $file;
            
            $this->fileUploader->uploadFilesInRow($row, 'FILE', 'FechamentosREINF');
            
            return $row['FILE'][0]['path'];
            
            /*
            $fileContent = file_get_contents($tmpFile);
            $fileBase64 = base64_encode($fileContent);
        
            $fileBase64Zeedhi = 'data: '.mime_content_type($tmpFile).';base64,'.$fileBase64;
            
            $row = [
                'FILE' => [
                    [
                        "name" => $fileName.'.txt', 
                        "type" => 'application/txt', 
                        "size" => strlen($fileContent), 
                        "path" => mime_content_type($tmpFile), 
                        "b64File" => $fileBase64Zeedhi
                    ]
                ],
                'FOLDER' => 'ObrigacoesAcessorias'
            ];
            return $this->fileUploader->uploadFile($row)[0]->getRows()[0]['FILE'][0]['PATH'];
            */

        } else {
            return false;
        }
    }
    
   /****************************************************
   *                Funções de Operações               *
   *                                                   *
   ****************************************************/
    public function salvarEvento($row){
        try{
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            $msgRetorno = '';
            $row        = $this->validacaoSaveEvento($row);
            // Alguns eventos nescessitam de outras operações antes de salvar
            switch ($row['NMEVENTO']) {
                case self::R1000:
                    $this->saveR1000($row);
                    break;
                case self::R2010:
                    $msgRetorno = $this->saveEventoContribuicoesPrevidenciarias($row);
                    break;
                case self::R2020:
                    $msgRetorno = $this->saveEventoContribuicoesPrevidenciarias($row);
                    break;
                case self::R2098:
                    $this->saveR2098($row);
                    break;    
                case self::R9000:
                    $this->saveR9000($row);
                    break;
                default:
                    $dataSet = new DataSet('envioeventoreinf', array($row));
                    $this->dataSourceManager->persist($dataSet);
                    $this->entityManager->flush();
                    break;
            }
            $connection->commit();
            return $msgRetorno;
        }catch(\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
    public function validacaoSaveEvento ($row){
        /*
        if($row['NMEVENTO'] ==='R-1000'){
            // Caso ja Exista o evento para o periodo informado inclui o de alteracao
            if($this->factory->verificaEventoPeriodo($row['NREVENTO'], $row['DTMESCOMPETENC']))
                $row ['IDOPERACAO'] = 'A';    
        }
        */
        return $row;
        
    }
    
    public function saveEventoContribuicoesPrevidenciarias($row){
        try {    
            $imposto = $row['NMEVENTO'] == self::R2010 ? '41' :'42';
            $dtFinal = DateUtil::getUltimoDiaDoMes( $this->formataData($row['DTMESCOMPETENC']))->format("d/m/Y");
            $mapa    = $this->factory->getMapaPeriodo($row['CDEMPRESA'], $row['DTMESCOMPETENC'], $dtFinal, $imposto);
            if($mapa){ 
                $participantes = $this->factory->getParticipantesMapa($mapa['NRSEQUMAPAPU']);
                if(count($participantes)>0){
                    $rowsEvento = [];
                    for($i=0; $i< count($participantes); $i++){
                        // Verifica se o evento do prestador ja existe no período informado;
                        $evento = $this->factory->verificaEventoServicos( $row['CDEMPRESA'], $row['DTMESCOMPETENC'], $row['NREVENTO'], $participantes[$i]['CDESTABREINF'] ) ;
                        if(!$evento){
                            // A primeira row já possui um sequencial.
                            if($i!==0)
                                $row ['NRENVIO'] = $this->newCode->getCodeByTableName('ENVIOEVENTOREINF', $this->nrorg,1,20);
                            
                            $row['NRSEQUMAPAPU'] = $mapa['NRSEQUMAPAPU'];
                            $row['CDESTABREINF'] = $participantes[$i]['CDESTABREINF'];
                            $rowsEvento[]        = $row;
                        }
                    }
                    if(!empty($rowsEvento)){
                        $dataSet = new DataSet('envioeventoreinf',$rowsEvento);
                        $this->dataSourceManager->persist($dataSet);
                        $this->entityManager->flush();
                        
                    }
                    
                    if(count($rowsEvento) == count($participantes))
                        return  $row['NMEVENTO'] == self::R2010 ?  MessagesFiscal::SUCESSO_2010 : MessagesFiscal::SUCESSO_2020;    
                    else
                        return  $row['NMEVENTO'] == self::R2010 ?  MessagesFiscal::EXISTE_EVT_2010 : MessagesFiscal::EXISTE_EVT_2020;        
                
                    
                }else{
                    throw new \Exception(MessagesFiscal::MAPA_VAZIO);
                }
            }else{
                throw new \Exception(MessagesFiscal::NAO_EXISTE_MAPA);
            }    
        } catch (\Exception $e) {
            throw $e;
        }
    }
   
    private function saveR9000($row) {
        $eventos = $row['NRENVIOEXCLUSAO'];
        $nrenvio = $row['NRENVIO'];
        
        foreach ($eventos as $evento) {
            //Pega dados do envio a ser excluido
            $params = array(
                'nrenvio' => $evento,
                'cdempresa' => $row['CDEMPRESA']
            );
            $envioParaExcluir = $this->entityManager->getRepository(Envioeventoreinf::class)->findOneBy($params);
            //Muda row para salvar evento de exclusão igual ao evento que deve ser excluido
            $row['DTMESCOMPETENC'] = $envioParaExcluir->getDtmescompetenc()->format('d/m/Y');
            $row['CDESTABREINF'] = $envioParaExcluir->getCdestabreinf();
            $row['NRENVIO'] = $nrenvio;
            $row['NRENVIOEXCLUSAO'] = $evento;  
            $dataSet = new DataSet('envioeventoreinf', array($row));
            $this->dataSourceManager->persist($dataSet);
            $this->entityManager->flush();
            $nrenvio ++;
        }
    }
    
    private function saveR2098($row) {
        $params = array(
            'DTMESCOMPETENC' => $row['DTMESCOMPETENC'],
            'CDEMPRESA' => $row['CDEMPRESA'],
            'IDAMBIENTE' => $row['IDAMBIENTE']
        );
        $envioPararReabrir = $this->factory->getR2099ParaReabrir($params);
        if ($envioPararReabrir) $row['NRENVIOEXCLUSAO'] = $envioPararReabrir['NRENVIO'];
        $dataSet = new DataSet('envioeventoreinf', array($row));
        $this->dataSourceManager->persist($dataSet);
        $this->entityManager->flush();
    }
    
    private function saveR1000($row) {
        if ($row['IDOPERACAO'] === 'E') {
            $row['NRENVIOEXCLUSAO'] = $this->nrenvioR1000vigente($row['CDEMPRESA'], $row['IDAMBIENTE'], $row['DTINIVALIDADE'], $row['DTFINVALIDADE']);
        } else if ($row['IDOPERACAO'] === 'I') {
            //$this->podeInserirR100($row['CDEMPRESA'], $row['IDAMBIENTE'], $row['DTINIVALIDADE'], $row['DTFINVALIDADE']);
        }
        $dataSet = new DataSet('envioeventoreinf', array($row));
        $this->dataSourceManager->persist($dataSet);
        $this->entityManager->flush();
    }
    
    
   /****************************************************
   *               Get Dados Eventos                   *
   *                                                   *
   ****************************************************/
   
   
    /*************** Evento R-1000 *********************/
    
    public function getDadosR1000($parametros) {
        try {
            return $this->factory->getDadosR1000($parametros['CDEMPRESA'], $parametros['DTINIVALIDADE'], $parametros['NRENVIO'], $parametros['IDAMBIENTE'], $parametros['DTFINVALIDADE']);
        } catch(\Exception $e) {
            throw $e;
        }
    }

    public function getDadosR1000softHouse() {
        $dados = $this->factory->getDadosR1000softHouse();
        return $dados;
    }
    
    public function getDadosR1000LimparBase($cdempresa) {
        $dados = $this->factory->getDadosR1000LimparBase($cdempresa);
        return $dados;
    }
    
    
    /*************** Evento R-1070 *********************/

    public function getDadosR1070($parametros) {
        try {
            return $this->factory->getDadosR1070($parametros['CDEMPRESA'], $parametros['NRENVIO'], $parametros['NREVENTO'], $parametros['DSVERSAO'], $parametros['IDAMBIENTE'], $parametros['CDPROCESSO']);
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function getDadosR1070infoSusp($dadosAgrupador, $dadosEnvio) {
        try {
            return $this->factory->getDadosR1070infoSusp($dadosEnvio['CDEMPRESA'], $dadosEnvio['NREVENTO'], $dadosEnvio['NRENVIO'], $dadosEnvio['CDPROCESSO']);
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    
    /*************** Evento R-2010 *********************/
    
    public function getDadosR2010($parametros) {
        try {
            //$cdestabreinf
            $parametroReinf = $this->getParamReinf($parametros['CDEMPRESA'])[0];
            //Informe [1] para arquivo original ou [2] para arquivo de retificação.
            $indretif       = $parametros['IDOPERACAO'] == 'R' ? '2' : '1';
            $dtFinal        = DateUtil::getUltimoDiaDoMes( $this->formataData($parametros['DTMESCOMPETENC']));
            //var_dump($parametros);die;
            return $this->factory->getDadosR2010($parametros['CDEMPRESA'], $parametros['DTMESCOMPETENC'], $dtFinal->format("d/m/Y"), $parametros['NRENVIO'], $parametroReinf['IDINDCPRB'], $parametros['DSVERSAO'], $parametros['NREVENTO'], $parametros['NRRECIBO'], $indretif, $parametros['CDESTABREINF'], $parametros['IDAMBIENTE'] );
        } catch(\Exception $e) {
            throw $e;
        }
    }

    public function getDadosR2010nfs($dadosAgrupador, $dadosEnvio) {
        try {
            return $this->factory->getDadosR2010nfs($dadosEnvio['CDEMPRESA'], $dadosEnvio['NREVENTO'], $dadosEnvio['NRENVIO'], $dadosEnvio['CDESTABREINF']);
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function getDadosR2010infoTpServ($dadosAgrupador, $dadosEnvio, $nota) {
        try {
            return $this->factory->getDadosR2010infoTpServ($dadosEnvio['CDEMPRESA'], $dadosEnvio['NREVENTO'], $dadosEnvio['NRENVIO'], $nota['NRLANCTONF'], $nota['CDFILIAL']);
        } catch(\Exception $e) {
            throw $e;
        }
    }
   
    /*
    public function getDadosR2010infoProcRetPr($dadosAgrupador, $dadosEnvio) {
        try {
            $dados   = $this->factory->getDadosR2010infoProcRetPr($dadosEnvio['CDEMPRESA'], $dadosEnvio['NREVENTO'], $dadosEnvio['NRENVIO']);
            return $dados;
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function getDadosR2010infoProcRetAd($dadosAgrupador, $dadosEnvio) {
        try {
            $dados   = $this->factory->getDadosR2010infoProcRetAd($dadosEnvio['CDEMPRESA'], $dadosEnvio['NREVENTO'], $dadosEnvio['NRENVIO']);
            return $dados;
        } catch(\Exception $e) {
            throw $e;
        }
    }
    */
    
    /*************** Evento R-2020 *********************/

    public function getDadosR2020($parametros) {
        try {
            $parametroReinf = $this->getParamReinf($parametros['CDEMPRESA'])[0];
            //Informe [1] para arquivo original ou [2] para arquivo de retificação.
            $indretif       = $parametros['IDOPERACAO'] == 'R' ? '2' : '1';
            $dtFinal        = DateUtil::getUltimoDiaDoMes( $this->formataData($parametros['DTMESCOMPETENC']));
            return $this->factory->getDadosR2020($parametros['CDEMPRESA'], $parametros['DTMESCOMPETENC'], $dtFinal->format("d/m/Y"), $parametros['NRENVIO'], $parametroReinf['IDINDCPRB'], $parametros['DSVERSAO'], $parametros['NREVENTO'], $parametros['NRRECIBO'], $indretif, $parametros['CDESTABREINF'], $parametros['IDAMBIENTE'] );
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function getDadosR2020nfs($dadosAgrupador, $dadosEnvio) {
        try {
            return $this->factory->getDadosR2020nfs($dadosEnvio['CDEMPRESA'], $dadosEnvio['NREVENTO'], $dadosEnvio['NRENVIO'], $dadosEnvio['CDESTABREINF']);
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function getDadosR2020infoTpServ($dadosAgrupador, $dadosEnvio, $nota) {
        try {
            return $this->factory->getDadosR2020infoTpServ($dadosEnvio['CDEMPRESA'], $dadosEnvio['NREVENTO'], $dadosEnvio['NRENVIO'], $nota['NRLANCTONF'], $nota['CDFILIAL']);
        } catch(\Exception $e) {
            throw $e;
        }
    }

    
    /*
    public function getDadosR2020infoProcRetPr($dadosAgrupador, $dadosEnvio) {
        try {
            $dados   = $this->factory->getDadosR2010infoProcRetPr($dadosEnvio['CDEMPRESA'], $dadosEnvio['NREVENTO'], $dadosEnvio['NRENVIO']);
            return $dados;
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function getDadosR2020infoProcRetAd($dadosAgrupador, $dadosEnvio) {
        try {
            $dados   = $this->factory->getDadosR2010infoProcRetAd($dadosEnvio['CDEMPRESA'], $dadosEnvio['NREVENTO'], $dadosEnvio['NRENVIO']);
            return $dados;
        } catch(\Exception $e) {
            throw $e;
        }
    }
    */
    
    /*************** Evento R-2060 *********************/
    
    public function getDadosR2060($parametros) {
        try {
            $parametroReinf = $this->getParamReinf($parametros['CDEMPRESA'])[0];
            
            $indretif       = $parametros['IDOPERACAO'] == 'R' ? '2' : '1';
            $dtInicial  = DateUtil::getPrimeiroDiaDoMes( $this->formataData($parametros['DTMESCOMPETENC']));
            $dtFinal    = DateUtil::getUltimoDiaDoMes( $this->formataData($parametros['DTMESCOMPETENC']));
            return $this->factory->getDadosR2060($parametros['CDEMPRESA'], $dtInicial->format("d/m/Y"), $dtFinal->format("d/m/Y"), $parametros['NRENVIO'], $parametros['NREVENTO'], $parametros['NRRECIBO'], $parametros['DSVERSAO'], $indretif, $parametros['IDAMBIENTE'], $parametroReinf['CDATIVIDADE'], $parametroReinf['VRALIQCPRB']);
        } catch(\Exception $e) {
            throw $e;
        }
    }
   
    public function getDadosR2060tipoCod($dadosAgrupador, $dadosEnvio) {
        try {
            return $this->factory->getDadosR2060tipoCod($dadosEnvio['CDEMPRESA'], $dadosEnvio['NREVENTO'], $dadosEnvio['NRENVIO'], $dadosAgrupador['NRINSCESTAB']);
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function getDadosR2060tipoAjuste($dadosAgrupador, $dadosEnvio) {
        try {
            return $this->factory->getDadosR2060tipoAjuste($dadosEnvio['CDEMPRESA'], $dadosEnvio['NREVENTO'], $dadosEnvio['NRENVIO'], $dadosAgrupador['CODATIVECON']);
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    /*************** Evento R-2098 *********************/
    
    public function getDadosR2098($parametros) {
        try {
            return $this->factory->getDadosR2098($parametros['CDEMPRESA'], $parametros['NRENVIO'], $parametros['NREVENTO'], $parametros['DSVERSAO'], $parametros['IDAMBIENTE'], $parametros['DTMESCOMPETENC']);
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    /*************** Evento R-2099 *********************/
    
    public function getDadosR2099($parametros) {
        try {
            return $this->factory->getDadosR2099($parametros['CDEMPRESA'], $parametros['NRENVIO'], $parametros['NREVENTO'], $parametros['DSVERSAO'], $parametros['IDAMBIENTE'], $parametros['DTMESCOMPETENC']);
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    /*************** Evento R-5011 *********************/
    
    public function getDadosR5011($parametros) {
        try {
            return $this->factory->getDadosR5011($parametros['NRENVIO'], $parametros['DSVERSAO']);
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    /*************** Evento R-9000 *********************/
    
    public function getDadosR9000($parametros) {
        try {
            return $this->factory->getDadosR9000($parametros['NRENVIO'], $parametros['NRENVIOEXCLUSAO'], $parametros['DSVERSAO']);
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function getInscricoesMunicipais($empresa) {
        return $this->factory->getInscricoesMunicipais($empresa);
    }
    
    public function getEventoR1000($versao) {
        return $this->factory->getEventoR1000($versao);
    }
    
    public function updateVersaoRetificacao($nrenvio, $nmevento ,$versao){
        $nrevento = $this->factory->selectEventoToUpdateRetificacao($nmevento ,$versao)['NREVENTO'];
        
        $this->factory->updateVersaoRetificacao($nrenvio, $versao, $nrevento);
    }
    
    public function decrypt($text, $salt = 'teknisa'){
        return trim(
            mcrypt_decrypt(
                MCRYPT_RIJNDAEL_256,
                str_pad($salt, 32, "\0"),
                base64_decode($text),
                MCRYPT_MODE_ECB,
                mcrypt_create_iv(mcrypt_get_iv_size(MCRYPT_RIJNDAEL_256, MCRYPT_MODE_ECB), MCRYPT_RAND)
            )
        );
    }
    
}